#ifndef CONTEXT_BBTUPLE_H_INCLUDED
#define CONTEXT_BBTUPLE_H_INCLUDED 1

#include "bbdata.h"
#include "concretedata.h"
#include "context_uids.h"
#include "bbtypes.h"

const TTypeName KTupleType = { { CONTEXT_UID_SENSORDATAFACTORY }, 7, 1, 0 };
const TTypeName KSubNameType = { { CONTEXT_UID_SENSORDATAFACTORY }, 8, 1, 0 };
const TTypeName KTupleMetaType = { { CONTEXT_UID_SENSORDATAFACTORY }, 9, 1, 0 };


_LIT(KStamp, "datetime");
_LIT(KPriority, "priority");
_LIT(KId, "id");
_LIT(KMeta, "tuplename");
_LIT(KModuleUid, "module_uid");
_LIT(KModuleId, "module_id");
_LIT(KSubName, "subname");



class TBBTupleSubName  : public TBBFixedLengthStringBase {
public:
	TBuf<128> iValue;
	IMPORT_C virtual const TTypeName& Type() const;
	IMPORT_C static const TTypeName& StaticType();

	IMPORT_C virtual TBool Equals(const MBBData* aRhs) const;

	TDes& operator()() { return iValue; }
	const TDesC& operator()() const { return iValue; }

	virtual TDes& Value() { return iValue; }
	virtual const TDesC& Value() const  { return iValue; }
	IMPORT_C MBBData* CloneL(const TDesC& Name) const;
public:
	TBBTupleSubName(const TDesC& aName) : TBBFixedLengthStringBase(aName) { }
	TBBTupleSubName(const TDesC& aValue, const TDesC& aName) : TBBFixedLengthStringBase(aName) { iValue=aValue.Left(128); }
	TBBTupleSubName& operator=(const TDesC& aValue) { iValue=aValue.Left(128); return *this; }
	bool operator==(const TBBTupleSubName& aRhs) const { return !(iValue.Compare(aRhs.iValue)); }
};

class TBBTupleMeta : public TBBCompoundData {
public:
	TBBUid			iModuleUid;
	TBBInt			iModuleId;
	TBBTupleSubName		iSubName;

	IMPORT_C virtual const TTypeName& Type() const;
	IMPORT_C virtual TBool Equals(const MBBData* aRhs) const;

	IMPORT_C static const TTypeName& StaticType();
	IMPORT_C const MBBData* Part(TUint aPartNo) const;

	IMPORT_C TBBTupleMeta();
	IMPORT_C TBBTupleMeta(TInt aModuleUid, TInt aModuleId,
		const TDesC& aSubName);

	IMPORT_C bool operator==(const TBBTupleMeta& aRhs) const;
	IMPORT_C virtual const TDesC& StringSep(TUint aBeforePart) const;
	IMPORT_C TBBTupleMeta& operator=(const TBBTupleMeta& aMetaData);
	IMPORT_C MBBData* CloneL(const TDesC& Name) const;
};

_LIT(KExpires, "expires");

class CBBTuple : public CBase, public TBBCompoundData {
public:
	TBBUint			iTupleId;
	TBBTupleMeta		iTupleMeta;
	TBBTime			iExpires;
	CBBGeneralHolder	iData;

	IMPORT_C virtual const TTypeName& Type() const;
	IMPORT_C virtual TBool Equals(const MBBData* aRhs) const;

	IMPORT_C static const TTypeName& StaticType();
	IMPORT_C const MBBData* Part(TUint aPartNo) const;

	IMPORT_C CBBTuple(MBBDataFactory* aFactory,
		TUint iTupleId=0, TInt aModuleUid=0, TInt aModuleId=0,
		const TDesC& aSubName=KNullDesC, MBBData* aData=0);

	IMPORT_C bool operator==(const CBBTuple& aRhs) const;
	IMPORT_C virtual const TDesC& StringSep(TUint aBeforePart) const;

	IMPORT_C CBBTuple& operator=(const CBBTuple& aTuple);
	IMPORT_C MBBData* CloneL(const TDesC& Name) const;
private:
	MBBData* GetPart(const TDesC& aName, const TTypeName& aType, TUint& aPartNoInto);
	MBBDataFactory*	iFactory;
	TBuf<50>	iValueName;
};

#endif
