#include "bbtuple.h"

EXPORT_C const TTypeName& TBBTupleSubName::Type() const
{
	return KSubNameType;
}

EXPORT_C const TTypeName& TBBTupleSubName::StaticType()
{
	return KSubNameType;
}

EXPORT_C TBool TBBTupleSubName::Equals(const MBBData* aRhs) const
{
	const TBBTupleSubName* rhs=bb_cast<TBBTupleSubName>(aRhs);
	return (rhs && *this==*rhs);
}

EXPORT_C const TTypeName& TBBTupleMeta::Type() const
{
	return KTupleMetaType;
}

EXPORT_C TBool TBBTupleMeta::Equals(const MBBData* aRhs) const
{
	const TBBTupleMeta* rhs=bb_cast<TBBTupleMeta>(aRhs);
	return (rhs && *this==*rhs);
}

EXPORT_C const TTypeName& TBBTupleMeta::StaticType()
{
	return KTupleMetaType;
}

EXPORT_C const MBBData* TBBTupleMeta::Part(TUint aPartNo) const
{
	switch(aPartNo) {
	case 0:
		return &iModuleUid;
	case 1:
		return &iModuleId;
	case 2:
		return &iSubName;
	default:
		return 0;
	}
}

EXPORT_C TBBTupleMeta::TBBTupleMeta() : TBBCompoundData(KMeta),
	iModuleUid(0, KModuleUid), iModuleId(0, KModuleId), iSubName(KNullDesC, KSubName)
{
}

EXPORT_C TBBTupleMeta::TBBTupleMeta(TInt aModuleUid, TInt aModuleId, const TDesC& aSubName) : TBBCompoundData(KMeta),
	iModuleUid(aModuleUid, KModuleUid), iModuleId(aModuleId, KModuleId), iSubName(aSubName, KSubName)
{
}

EXPORT_C bool TBBTupleMeta::operator==(const TBBTupleMeta& aRhs) const
{
	return (
		iModuleUid==aRhs.iModuleUid &&
		iModuleId==aRhs.iModuleId &&
		iSubName==aRhs.iSubName 
		);
}

_LIT(KSpace, " ");

EXPORT_C const TDesC& TBBTupleMeta::StringSep(TUint aBeforePart) const
{
	if (aBeforePart==0 || aBeforePart > 2) return KNullDesC;
	return KSpace;
}


EXPORT_C const TTypeName& CBBTuple::Type() const
{
	return KTupleType;
}

EXPORT_C TBool CBBTuple::Equals(const MBBData* aRhs) const
{
	const CBBTuple* rhs=bb_cast<CBBTuple>(aRhs);
	return (rhs && *this==*rhs);
}

EXPORT_C const TTypeName& CBBTuple::StaticType()
{
	return KTupleType;
}

EXPORT_C const MBBData* CBBTuple::Part(TUint aPartNo) const
{
	switch(aPartNo) {
	case 0:
		return &iTupleId;
	case 1:
		return &iTupleMeta;
	case 2:
		return &iData;
	case 3:
		return &iExpires;
	default:
		return 0;
	}
}

_LIT(KTuple, "tuple");
_LIT(KValue, "tuple_value");

EXPORT_C CBBTuple::CBBTuple(MBBDataFactory* aFactory, TUint aTupleId, TInt aModuleUid, TInt aModuleId,
			    const TDesC& aSubName, MBBData* aData) : TBBCompoundData(KTuple),
			    iTupleId(aTupleId, KId),
			    iTupleMeta(aModuleUid, aModuleId, aSubName), iExpires(KExpires),
			    iData(KValue, aFactory, aData), iFactory(aFactory)
{
}

EXPORT_C bool CBBTuple::operator==(const CBBTuple& aRhs) const
{
	return (
		iTupleId==aRhs.iTupleId &&
		iTupleMeta==aRhs.iTupleMeta &&
		iExpires == aRhs.iExpires &&
		iData.Equals(aRhs.iData())
		);
}

EXPORT_C const TDesC& CBBTuple::StringSep(TUint aBeforePart) const
{
	if (aBeforePart==0 || aBeforePart > 3) return KNullDesC;
	return KSpace;
}

MBBData* CBBTuple::GetPart(const TDesC& aName, const TTypeName& aType, TUint& aPartNoInto)
{
	MBBData* p=TBBCompoundData::GetPart(aName, aType, aPartNoInto);
	if (!p || aName.Compare(KValue)==0 ) {
		if (iExpires.Name().Compare(aName)==0) {
			aPartNoInto=3;
			return &iExpires;
		}
		iValueName=aName;
		iData.SetValue(iFactory->CreateBBDataL(aType, iValueName, iFactory));
		aPartNoInto=2;
		p=&iData;
	}
	return p;
}

EXPORT_C MBBData* TBBTupleSubName::CloneL(const TDesC& Name) const
{
	return new (ELeave) TBBTupleSubName(iValue, Name);
}

EXPORT_C TBBTupleMeta& TBBTupleMeta::operator=(const TBBTupleMeta& aMetaData)
{
	iModuleUid()=aMetaData.iModuleUid();
	iModuleId()=aMetaData.iModuleId();
	iSubName()=aMetaData.iSubName();
	return *this;
}

EXPORT_C MBBData* TBBTupleMeta::CloneL(const TDesC&) const
{
	return new (ELeave) TBBTupleMeta(iModuleUid(), iModuleId(), iSubName());
}

EXPORT_C CBBTuple& CBBTuple::operator=(const CBBTuple& aTuple)
{
	MBBData* val=0;
	if (aTuple.iData()) {
		val=aTuple.iData()->CloneL(aTuple.iData()->Name());
	}
	iData.SetValue(val);
	iData.SetOwnsValue(ETrue);
	iTupleId()=aTuple.iTupleId();
	iTupleMeta=aTuple.iTupleMeta;
	iExpires()=aTuple.iExpires();

	return *this;
}
	
EXPORT_C MBBData* CBBTuple::CloneL(const TDesC& ) const
{
	auto_ptr<CBBTuple> ret(new (ELeave) CBBTuple(iFactory));
	*ret=*this;
	return ret.release();
}


