#include "alerter.h"
#include "app_context.h"
#include "settings.h"
#include "symbian_auto_ptr.h"

#include <MdaAudioSamplePlayer.h>

#define SETTING_VIBRATE_ONLY	27
#include <eikenv.h>

#include <profileapi.h>
#ifndef NO_PROFILEAPI_H
#else
typedef TInt32 TContactItemId;
class CProfileAPI : public CBase {
public:
	enum TProErrorCode
	    {
	    EPro0=0
	   ,EPro1
	    };
	IMPORT_C static CProfileAPI* NewL(TBool);
	IMPORT_C virtual ~CProfileAPI();
	IMPORT_C TProErrorCode GetProfileActiveName(TPtr, TInt*);
	IMPORT_C TProErrorCode GetProfileMultiData( TDes&, TDes&, TInt&,
					       TInt&, TBool&, TInt&, CArrayFixFlat<TContactItemId>*,
					       TInt&, TInt);
};
#endif

class CAlerterImpl : public CAlerter, public MContextBase, public MMdaAudioPlayerCallback
{
private:
	CAlerterImpl(MApp_context& Context);
	void ConstructL();

	virtual void ShortAlert();
	virtual void AlertUser();
	virtual void StopAlert();
	~CAlerterImpl();

	virtual void PlayAlert(TBool aShort);
	virtual void MapcInitComplete(TInt aError, 
		const TTimeIntervalMicroSeconds& aDuration);
	virtual void MapcPlayComplete(TInt aError);

	int get_current_profile();
	CMdaAudioPlayerUtility* iPlayer;
	bool	ringing;
	
	friend class CAlerter;
	friend class auto_ptr<CAlerterImpl>;
};

EXPORT_C CAlerter* CAlerter::NewL(MApp_context& Context)
{
	auto_ptr<CAlerterImpl> ret(new (ELeave) CAlerterImpl(Context));
	ret->ConstructL();
	return ret.release();
}

CAlerterImpl::CAlerterImpl(MApp_context& Context) : MContextBase(Context)
{
}

void CAlerterImpl::ConstructL()
{
}

int CAlerterImpl::get_current_profile()
{
#ifndef __WINS__
	auto_ptr<CProfileAPI> profile(CProfileAPI::NewL(EFalse));
	auto_ptr<HBufC> name(HBufC::NewL(30));
	TPtr16 p=name->Des();

	TInt id;
	if (profile->GetProfileActiveName(p, &id)!= 0)
		return -1; // Get active profile

	if (id<0 || id>10) return -1;

	return id;
#else
	return 0;
#endif
}

void CAlerterImpl::AlertUser()
{
	PlayAlert(EFalse);
}

void CAlerterImpl::ShortAlert()
{
	PlayAlert(ETrue);
}


void CAlerterImpl::PlayAlert(TBool aShort)
{
#ifndef __WINS__
	if (iPlayer) return;
	TFileName tone=DataDir();

	int profile=get_current_profile();

	switch (profile) {
	case 2:
		// meeting
		if (!aShort) {
			tone.Append(_L("silent.rng"));
		} else {
			tone.Append(_L("shortsilent.rng"));
		}
		break;
	case 1:
		return;
		// silent
		break;
	default:
		{
		// general, outdoor
		TBool only_vibrate=true;
		Settings().GetSettingL(SETTING_VIBRATE_ONLY, only_vibrate);
		if (only_vibrate) {
			if (!aShort) {
				tone.Append(_L("silent.rng"));
			} else {
				tone.Append(_L("shortsilent.rng"));
			}
		} else {
			if (!aShort) {
				tone.Append(_L("tone.rng"));
			} else {
				tone.Append(_L("shorttone.rng"));
			}
		}		
		break;
		}
	}

	iPlayer=CMdaAudioPlayerUtility::NewFilePlayerL(tone, *this,
		EMdaPriorityMax,(TMdaPriorityPreference)0x01340001); /* EAknAudioPrefIncomingCall */
#endif
}

void CAlerterImpl::StopAlert()
{
#ifndef __WINS__
	if (ringing) {
		iPlayer->Stop();
		ringing=false;
	} else {
		delete iPlayer; iPlayer=0;
	}
#endif
}

CAlerterImpl::~CAlerterImpl()
{
	delete iPlayer; iPlayer=0;
}
void CAlerterImpl::MapcInitComplete(TInt aError, const TTimeIntervalMicroSeconds& aDuration)
{
	if (aError==KErrNone) {
		iPlayer->SetVolume(iPlayer->MaxVolume()*0.75);
		iPlayer->Play();
		ringing=true;
	} else {
		delete iPlayer; iPlayer=0;
		ringing=false;
	}
}
void CAlerterImpl::MapcPlayComplete(TInt aError)
{
	ringing=false;
	delete iPlayer; iPlayer=0;
}