#include "cc_schedulers.h"

#include "callstack.h"
#include "reporting.h"
#include "app_context_impl.h"
#include "errorhandling.h"

EXPORT_C CDebugAppScheduler::CDebugAppScheduler(MApp_context& ctx, CCoeEnv* aCoeEnv) : 
	CCoeScheduler(aCoeEnv), MContextBase(ctx) { }

class CActiveSchedulerAccess : public CBase
        {
public:
        IMPORT_C CActiveSchedulerAccess();
        IMPORT_C ~CActiveSchedulerAccess();
        IMPORT_C static void Install(CActiveScheduler* aScheduler);
        IMPORT_C static CActiveScheduler* Current();
        IMPORT_C static void Add(CActive* anActive);
        IMPORT_C static void Start();
        IMPORT_C static void Stop();
        IMPORT_C static TBool RunIfReady(TInt& aError, TInt aMinimumPriority);
        IMPORT_C static CActiveScheduler* Replace(CActiveScheduler* aNewActiveScheduler);
        IMPORT_C virtual void WaitForAnyRequest();
        IMPORT_C virtual void Error(TInt anError) const;
private:
        void DoStart();
        void OwnedStartLoop(TInt& aRunning);
        IMPORT_C virtual void OnStarting();
        IMPORT_C virtual void OnStopping();
        IMPORT_C virtual void Reserved_1();
        IMPORT_C virtual void Reserved_2();
private:
        // private interface used through by CActiveSchedulerWait objects
        friend class CActiveSchedulerWait;
        static void OwnedStart(CActiveSchedulerWait& aOwner);
protected:
        inline TInt Level() const;
public:
        TInt iLevel;
        TPriQue<CActive> iActiveQ;
        };

void CDebugAppScheduler::WaitForAnyRequest() 
{
	MActiveErrorReporter* rep=AppContext().GetActiveErrorReporter();
	if (rep) rep->SetInHandlableEvent(EFalse);
	CallStackMgr().ResetCallStack();
	ErrorInfoMgr().ResetLastErrorInfo();
	CCoeScheduler::WaitForAnyRequest();
}

#ifdef __WINS__
EXPORT_C void CDebugAppScheduler::CheckThatEmpty()
{
	CActiveSchedulerAccess* a=(CActiveSchedulerAccess*)(CActiveScheduler*)this;
	TDblQueIter<CActive> iterator(a->iActiveQ);
	for (CActive* active=iterator++; active; active=iterator++) {
		CCheckedActive* ca=(CCheckedActive*)active;
		TInt x;
		x=0;
	}
}
#endif

EXPORT_C CDebugBaScheduler::CDebugBaScheduler(MApp_context& ctx) : 
	CBaActiveScheduler(), MContextBase(ctx) { }

void CDebugBaScheduler::WaitForAnyRequest() 
{
	MActiveErrorReporter* rep=AppContext().GetActiveErrorReporter();
	if (rep) rep->SetInHandlableEvent(EFalse);
	CallStackMgr().ResetCallStack();
	ErrorInfoMgr().ResetLastErrorInfo();
	CBaActiveScheduler::WaitForAnyRequest();
}
