#include "JaikuContactsListbox.h"
 
#include "phonebook.h"
#include "presence_ui_helper.h"

// From context
#include <icons.h>
#include <contextcommon.mbg>
#include <pointer.h>
#include <reporting.h>

// From S60
#include <AknIconArray.h>
#include <aknlists.h>
#include <avkon.hrh>
#include <avkon.mbg>
#include <eikenv.h>
#include <eikfrlb.h>
#include <eikfrlbd.h>
#include <eiklbi.h>
#include <gdi.h>

// 
const TInt KNameSubCellIx = 0;
const TInt KPresenceSubCellIx= 1;
const TInt KStatusIconSubCellIx = 2;

////////////////
// ListBox model

const MDesCArray* CJaikuContactsLBModel::MatchableTextArray() const
    {
    return iFilterArray;
    }

EXPORT_C void CJaikuContactsLBModel::SetFilterArray( MDesCArray* aFilterArray )
    {
    iFilterArray = aFilterArray;
    }


EXPORT_C CJaikuContactsLBModel* CJaikuContactsLBModel::NewL()
	{
	CALLSTACKITEMSTATIC_N( _CL("CJaikuContactsLBModel"), _CL("NewL") );
	auto_ptr<CJaikuContactsLBModel> self( new (ELeave) CJaikuContactsLBModel );
	self->ConstructL();
	return self.release();
	}




////////////////
// Icons

const TInt KNbIcons = 5;
static const TIconID iconId[KNbIcons]=
{
// System 
     _INIT_T_ICON_ID("Z:\\system\\data\\avkon.mbm", EMbmAvkonQgn_indi_marked_add, EMbmAvkonQgn_indi_marked_add_mask),
// Jaiku 
#ifndef __WINS__
    _INIT_T_ICON_ID("C:\\system\\data\\contextcommon.mbm", EMbmContextcommonLight_green,  EMbmContextcommonLight_mask ), 
    _INIT_T_ICON_ID("C:\\system\\data\\contextcommon.mbm", EMbmContextcommonLight_red,    EMbmContextcommonLight_mask ), 
    _INIT_T_ICON_ID("C:\\system\\data\\contextcommon.mbm", EMbmContextcommonLight_yellow, EMbmContextcommonLight_mask ), 
    _INIT_T_ICON_ID("C:\\system\\data\\contextcommon.mbm", EMbmContextcommonLight_gray,   EMbmContextcommonLight_mask ), 
#else
    _INIT_T_ICON_ID("Z:\\system\\data\\contextcommon.mbm", EMbmContextcommonLight_gray,   EMbmContextcommonLight_mask ), 
    _INIT_T_ICON_ID("Z:\\system\\data\\contextcommon.mbm", EMbmContextcommonLight_green,  EMbmContextcommonLight_mask ), 
    _INIT_T_ICON_ID("Z:\\system\\data\\contextcommon.mbm", EMbmContextcommonLight_red,    EMbmContextcommonLight_mask ), 
    _INIT_T_ICON_ID("Z:\\system\\data\\contextcommon.mbm", EMbmContextcommonLight_yellow, EMbmContextcommonLight_mask ), 
#endif
};

// #ifndef __WINS__
// 	_INIT_T_ICON_ID("C:\\system\\data\\contextcommon.mbm", EMbmContextcommonSpeaker_on, EMbmContextcommonSpeaker_on_mask ),
// 	_INIT_T_ICON_ID("C:\\system\\data\\contextcommon.mbm", EMbmContextcommonSpeaker_on_grey, EMbmContextcommonSpeaker_on_grey_mask ),
// 	_INIT_T_ICON_ID("C:\\system\\data\\contextcommon.mbm", EMbmContextcommonSpeaker_off, EMbmContextcommonSpeaker_off_mask ),
// 	_INIT_T_ICON_ID("C:\\system\\data\\contextcommon.mbm", EMbmContextcommonSpeaker_off_grey, EMbmContextcommonSpeaker_off_grey_mask ),
// #else // 
// 	_INIT_T_ICON_ID("Z:\\system\\data\\contextcommon.mbm", EMbmContextcommonSpeaker_on, EMbmContextcommonSpeaker_on_mask ),
//  	_INIT_T_ICON_ID("Z:\\system\\data\\contextcommon.mbm", EMbmContextcommonSpeaker_on_grey, EMbmContextcommonSpeaker_on_grey_mask ),
// 	_INIT_T_ICON_ID("Z:\\system\\data\\contextcommon.mbm", EMbmContextcommonSpeaker_off, EMbmContextcommonSpeaker_off_mask ),
// 	_INIT_T_ICON_ID("Z:\\system\\data\\contextcommon.mbm", EMbmContextcommonSpeaker_off_grey, EMbmContextcommonSpeaker_off_grey_mask ),
// #endif 
// };

void LoadJaikuIcons(CArrayPtrFlat<CGulIcon> * aIconList)
    {
    //CALLSTACKITEMSTATIC_N( _CL("CJaikuContactsListBox"), _CL("LoadJaikuIcons") );
    LoadIcons(aIconList, iconId, KNbIcons);
    }

TInt GetJaikuIconIndex(TInt identifier)
    {
    //CALLSTACKITEMSTATIC_N( _CL("CJaikuContactsListBox"), _CL("GetJaikuIconIndex"));

    return GetIconIndex(identifier, iconId, KNbIcons);
    }


////////////////
// Helper functions to create listbox item texts


_LIT( KPresenceIn_, "In "); 
_LIT( KPresence_in_, " in "); 
_LIT( KComma, ", "); 
_LIT( KSpace, " "); 

const TInt KPresenceMaxLen = 20;

void AppendPresenceTextL( TPtr& aBuf, contact& aContact )
    {
    CBBPresence* p = aContact.presence;

    // 2.a) No presence 
    if ( !p || ! p->iSent )
	{
	// leave empty
	}
    // 2.b) Has presence 
    else
	{	    
	//
	TBool hasPresence = EFalse;

	TPtrC msg( p->iUserGiven.iDescription.iValue );
	if ( msg.Length() )
	    {
	    hasPresence = ETrue;
	    aBuf.Append( msg.Left( KPresenceMaxLen ) );
	    }
	else
	    {
	    const TBBCalendar& cal = p->iCalendar;
	    TPtrC calEvent( cal.iCurrent.iDescription() );
	    if ( calEvent.Length() ) 
		{
		hasPresence = ETrue;
		aBuf.Append( calEvent.Left( KPresenceMaxLen ) );
		}
	    }
	
	TPtrC location;
	if ( p->iBaseInfo.iCurrent.IsSet() )
	    {
	    location.Set( p->iBaseInfo.iCurrent.iBaseName() );	    
	    }
	else if ( p->iCellName().Length() )
	    {
	    location.Set(  p->iCellName() );
	    }
	else 
	    {
	    TPtrC city;
	    TPtrC country;	
	city.Set( p->iCity() );
	country.Set( p->iCountry() );
	    if ( city.Length() )
		location.Set( city );
	    else 
		location.Set( country );
	    }

	TBool hasLoc = location.Length();

	if ( hasPresence && hasLoc )
	    {
	    aBuf.Append( KPresence_in_ );
	    }
	else if ( hasLoc )
	    {
	    aBuf.Append( KPresenceIn_ );
	    }
	
	if ( hasLoc  )
	    {
	    aBuf.Append( location );
	    }	
	}
    }

void AppendStatusIconL( TPtr& aBuf, contact& aContact )
    {
    CBBPresence* p = aContact.presence;

    // Icon
    TInt ix = KErrNotFound;
    if ( !p || ! p->iSent )
	{
	ix = GetJaikuIconIndex(EMbmContextcommonLight_gray);
	}
    else
	{
	TTime stamp = p->iSentTimeStamp();
	TBool outOfDate = IsOutOfDate( stamp, GetTime() );
	
	// out of date 
	
	if ( outOfDate )
	    {
	    ix = GetJaikuIconIndex(EMbmContextcommonLight_gray);
	    }
	else
	    {
	TInt ringType = p->iProfile.iRingingType();
	TBool vibraOn = p->iProfile.iVibra();
	TInt volumeOn = p->iProfile.iRingingVolume() != 0;
	
	// RED 
	if ( ( ringType == TBBProfile::ERingingTypeSilent || ! volumeOn ) && ! vibraOn )
	    {
	    ix = GetJaikuIconIndex(EMbmContextcommonLight_red);
	    }

	// ORANGE 
	else if ( (volumeOn && ringType == TBBProfile::ERingingTypeBeepOnce) || ( vibraOn && ! volumeOn ) )
	    {
	    ix = GetJaikuIconIndex(EMbmContextcommonLight_yellow);
	    }
	// GREEN 
	else
	    {
	    ix = GetJaikuIconIndex(EMbmContextcommonLight_green);
	    }
	}
	}
    if ( ix >= 0 )
	{
	TBuf<4> icon;
	icon.Format( _L("%02d"), ix );
	aBuf.Append( icon );
	}
    }

void AppendSelectionMarkerL( TPtr aBuf )
    {
    TBuf<4> icon;
    icon.Format(_L("%02d"), GetJaikuIconIndex(EMbmAvkonQgn_indi_marked_add) );
    aBuf.Append(icon);
    }


////////////////
// Custom array for jaiku listbox 


class CJaikuContactsArray : public CBase,
			    public MDesCArray,
			    public MContextBase
{
public: 
    CJaikuContactsArray( phonebook_i& aPhonebook ) 
	: iPhonebook( aPhonebook ), iShowLastNameFirst(ETrue) {
	
    }

void ConstructL( )
    {
    iBuf = HBufC::NewL( 1000 );

#ifdef __S60V2__
    CPbkContactEngine *eng = CPbkContactEngine::Static();
    CPbkContactEngine::TPbkNameOrder order;
    if (!eng) 
	{
	order = CPbkContactEngine::EPbkNameOrderLastNameFirstName;
	}
    else
	{
	order = eng->NameDisplayOrderL();
	}

    if (order != CPbkContactEngine::EPbkNameOrderLastNameFirstName) 
	{
	iShowLastNameFirst = EFalse;
	}
#endif
    
    }
~CJaikuContactsArray()
    {
    delete iBuf;
    }
private: // from MDesCArray
    TPtrC MdcaPoint(TInt aIndex) const;
    TInt MdcaCount() const;

private:
    /**
     * Internal buffer to format contact text
     * own. 
     */ 
    mutable HBufC* iBuf;
    TBool iShowLastNameFirst; 
    phonebook_i& iPhonebook;
};

TInt CJaikuContactsArray::MdcaCount() const
{
    CALLSTACKITEM_N( _CL("CJaikuContactsArray"), _CL("MdcaCount") );
    return iPhonebook.Count();
}

_LIT( KTab, "\t" );
TPtrC CJaikuContactsArray::MdcaPoint(TInt aIndex) const
{
    CALLSTACKITEM_N( _CL("CJaikuContactsArray"), _CL("MdcaPoint") );
    contact* c = iPhonebook.GetContact( aIndex );
    
    TPtr ptr = iBuf->Des();
    ptr.Zero();
    // 1) no contact found, create empty list item
    if ( !c )
	{
	
	_LIT(KEmptyName, " ");
	ptr.AppendFormat( _L("%S\t\t"), &KEmptyName());
	}
    
    // 2) Jaiku contact
    else if ( c->has_nick )
	{
	// Name 
	c->Name( iBuf, iShowLastNameFirst ); 
	ptr = iBuf->Des();
	ptr.Append( KTab );
	AppendPresenceTextL( ptr, *c );
	ptr.Append( KTab );
	AppendStatusIconL( ptr, *c );
	}

    // 3) Normal contact
    else  
	{
	c->Name( iBuf, iShowLastNameFirst );
	ptr = iBuf->Des();
	ptr.Append( _L("\t\t") );
	}
    
    // FIXME: teemu. const_cast hack to make reporting possible
    const_cast<CJaikuContactsArray*>(this)->Reporting().DebugLog( *iBuf );
    return *iBuf;
}

////////////////
// custom item drawer

class CJaikuLBItemDrawer : public CFormattedCellListBoxItemDrawer,
			   public MContextBase
    {
public:
    CJaikuLBItemDrawer( MTextListBoxModel* aModel,
			const CFont* aFont,
			CFormattedCellListBoxData* aLBData,
			CJaikuContactsListBox* aListBox,
			phonebook_i& aPhonebook )
	: CFormattedCellListBoxItemDrawer( aModel, aFont, aLBData ),
	  iPhonebook( aPhonebook )	
	{	
	}

    void DrawItemText( TInt aItemIndex, 
		       const TRect& aItemTextRect,
		       TBool aItemIsCurrent,
		       TBool aViewIsEmphasized,
		       TBool aItemIsSelected ) const
	{
	CALLSTACKITEM_N(_CL("CJaikuLBItemDrawer"), _CL("DrawItemText"));
	CFormattedCellListBoxData* lbData = FormattedCellData();
	const_cast<CJaikuLBItemDrawer*>(this)->Reporting().DebugLog( _L("DrawItemText "));
	
	TBool outOfDate = EFalse;
	contact* con = iPhonebook.GetContact( aItemIndex );
	if (  con && con->presence )
	    {
	    TTime stamp = con->presence->iSentTimeStamp();
	    outOfDate = IsOutOfDate( stamp, GetTime() );
	    }
	
	CFormattedCellListBoxData::TColors c = lbData->SubCellColors( KPresenceSubCellIx );
	if ( outOfDate )
	    {
	    // Set colors gray	    
	    TRgb grayed( 128,128,128 );
	    c.iText = grayed;
	    c.iHighlightedText = grayed;
	    lbData->SetSubCellColorsL(1, c);
	    }
	else
	    {
	    // We need to set colors back to "default", because previous
	    // items affect this. 
	    TRgb black( 0,0,0 );
	    c.iText = black;
	    c.iHighlightedText = black;
	    }
	lbData->SetSubCellColorsL(1, c);	    
	
	CFormattedCellListBoxItemDrawer::DrawItemText( aItemIndex,
						       aItemTextRect,
						       aItemIsCurrent,
						       aViewIsEmphasized,
						       aItemIsSelected );

	}
private:
    phonebook_i& iPhonebook;
    };


////////////////
// Custom jaiku listbox


/**
 * We need this only because we need to create own text list box model.
 * It also helps with debugging, because it derives from MContextBase
 */

void CJaikuContactsListBox::CreateItemDrawerL()
    {
    CALLSTACKITEM_N( _CL("CJaikuContactsListBox"), _CL("CreateItemDrawerL") );
    // Replace model
    delete iModel;
    iModel = NULL;
    iModel = CJaikuContactsLBModel::NewL();

    // Create item drawer
    CFormattedCellListBoxData* lbData = CFormattedCellListBoxData::NewL();    
    iItemDrawer = new (ELeave) CJaikuLBItemDrawer( Model(),
						   iEikonEnv->NormalFont(),
						   lbData,
						   this,
						   iPhonebook );

    }


EXPORT_C CJaikuContactsLBModel* CJaikuContactsListBox::JaikuModel() const
	{
	return static_cast<CJaikuContactsLBModel*>( Model() );
	}


EXPORT_C TRect CJaikuContactsListBox::AdjustAndSetRect( TRect aRect )
    {
    CALLSTACKITEM_N( _CL("CJaikuContactsListBox"), _CL("AdjustAndSetRect"));
    TRect lbRect = aRect;
    AdjustRectHeightToWholeNumberOfItems( lbRect );
    SetRect( lbRect );    
    return lbRect;
    }


void CJaikuContactsListBox::SizeChanged()
    {
    CALLSTACKITEM_N( _CL("CJaikuContactsListBox"), _CL("SizeChanged"));
    CEikFormattedCellListBox::SizeChanged();
    SetupSubcellsL();
    }


CJaikuContactsListBox::CJaikuContactsListBox( phonebook_i& aPhonebook )
    : iPhonebook( aPhonebook )
    {
    }


// Layouts
const TInt KRowHeight = 17;
const TInt KItemHeight = 2 * KRowHeight;



EXPORT_C CJaikuContactsListBox* CJaikuContactsListBox::CreateListboxL( 
    CCoeControl* aParent, 
    phonebook_i& aPhonebook )
{ 
    CALLSTACKITEMSTATIC_N( _CL("CJaikuContactsListBox"), _CL("CreateJaikuContactsListboxL"));

    // Create list box 
    auto_ptr<CJaikuContactsListBox> lb( new (ELeave) CJaikuContactsListBox(aPhonebook) );
    lb->ConstructL( aParent, EAknListBoxMarkableList ); 
    
    auto_ptr<CAknIconArray> iconArray( new (ELeave) CAknIconArray(10) );
    LoadJaikuIcons( iconArray.get() );
    lb->ItemDrawer()->FormattedCellData()->SetIconArray( iconArray.get() );
    iconArray.release();

    lb->SetItemHeightL( KItemHeight );
    lb->View()->SetListEmptyTextL( _L("(DEV: Empty)") );
    lb->CreateScrollBarFrameL( ETrue );
    lb->ScrollBarFrame()->SetScrollBarVisibilityL( CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto);
    
    return lb.release();
    }





/**
 *
 * subcells
 * 0 Contact Name text
 * 1 Presence info text
 * 2 Presence status icon (0 green, 1 orange, 2 red)
 *
 */

EXPORT_C MDesCArray* CJaikuContactsListBox::CreateLBArrayL( 
    phonebook_i& aPhonebook)
{
    CALLSTACKITEMSTATIC_N( _CL("CJaikuContactsListBox"), _CL("CreateJaikuContactsArrayL"));
    auto_ptr<CJaikuContactsArray> array( new (ELeave) CJaikuContactsArray( aPhonebook ) );
    array->ConstructL();
    return array.release();
}


void CJaikuContactsListBox::SetupSubcellsL(  ) 
{
    CALLSTACKITEM_N(_CL("CJaikuContactsListBox"), _CL("SetupSubcellsL"));
    CFormattedCellListBoxItemDrawer* itemDrawer = ItemDrawer();

    AknListBoxLayouts::SetupStandardListBox( *this );
    AknListBoxLayouts::SetupStandardFormListbox( itemDrawer );    
    
    const CFont* font = CEikonEnv::Static()->NormalFont();
    TInt subCell =  0;    

    TSize itemSize = View()->ItemSize();

    // Margins prevent drawing on top of focus borders 
    TMargins m = { 4, 4, 1, 1 };
    TSize sz = itemSize - TSize( m.iLeft, m.iTop ) - TSize( m.iRight, m.iBottom );
    
    TPoint p0, p1;
    TInt w,h;
    

    p0 = TPoint( 24, m.iTop );
    w = 176 - 24 - m.iRight; //itemSize.iWidth - p0.iX - m.iRight;
    h = sz.iHeight / 2;
    p1 = p0 + TSize( w, h );							  
    TInt b = (3*h) / 4;


    AknListBoxLayouts::SetupFormTextCell( *this, itemDrawer, subCell, 
                                          font, 
                                          215, // C
                                          p0.iX, // lm
                                          -1, // rm but ignored
                                          p0.iY + b, // B
                                          w,      // w
                                          CGraphicsContext::ELeft, 
                                          p0,
                                          p1 );
    //itemDrawer->FormattedCellData()->SubCellsMightIntersect( EFalse );
    //itemDrawer->FormattedCellData()->SetNotAlwaysDrawnSubCellL( subCell, ETrue );

    subCell++;   

    font = CEikonEnv::Static()->DenseFont();
    p0 += TSize( 0, h );
    p1 += TSize( 0, h );
    AknListBoxLayouts::SetupFormTextCell( *this, itemDrawer, subCell, 
                                          font, 
                                          215, // C
                                          p0.iX, // lm
                                          -1, // rm but ignored
                                          p0.iY + b,
                                          w,
                                          CGraphicsContext::ELeft, 
                                          p0,
                                          p1 );
    //itemDrawer->FormattedCellData()->SubCellsMightIntersect( EFalse );
    itemDrawer->FormattedCellData()->SetNotAlwaysDrawnSubCellL( subCell, ETrue );
    
    subCell++;
    w = 15;
    h = 15; 
    p0 = TPoint( m.iLeft /*+ sz.iWidth - w*/, m.iTop );
    p1 = p0 + TSize( w, h );
    AknListBoxLayouts::SetupFormGfxCell( *this, itemDrawer, subCell, 
                                         p0.iX, p0.iY, // l,t
                                         -1, -1, // r,b but ignored
                                         w, // W
                                         h, // H
                                         p0,
                                         p1 );
    //itemDrawer->FormattedCellData()->SubCellsMightIntersect( EFalse );
    itemDrawer->FormattedCellData()->SetNotAlwaysDrawnSubCellL( subCell, ETrue );

}

