/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/

/*
 * Concepts:
 * !Making calls!
 */

#include "break.h"
#include "phonehelper_ui.h"
#include "app_context.h"
#include <txtrich.h>
#include <SenduiMtmUids.h>
#include <eikenv.h>
#include <rpbkviewresourcefile.h> 
#include <cpbkcontacteditordlg.h> 
#include <MsgBioUids.h>
#include <msvapi.h>
#include <msvids.h>
#include <aknviewappui.h> 
#include <cpbkcontactitem.h> 
#include <bcardeng.h>
#include <CPbkContactEngine.h>

EXPORT_C void phonehelper_ui::ConstructL()
{
	CALLSTACKITEM_N(_CL("phonehelper_ui"), _CL("ConstructL"));
	phonehelper::ConstructL();

	iEngine=CPbkContactEngine::Static();
	if (iEngine) {
		owns_engine=EFalse;
	} else {
		iEngine=CPbkContactEngine::NewL();
		owns_engine=ETrue;
	}
}

EXPORT_C phonehelper_ui::~phonehelper_ui()
{
	CALLSTACKITEM_N(_CL("phonehelper_ui"), _CL("~phonehelper_ui"));
	if (owns_engine) delete iEngine;

}


EXPORT_C void phonehelper_ui::show_editor(TInt contact_id,  bool duplicate, TInt focus_field_index, CPbkFieldInfo * fieldInfo, const TDesC& no )
{
	CALLSTACKITEM_N(_CL("phonehelper_ui"), _CL("show_editor"));

	CCoeEnv *env = CEikonEnv::Static();
	RPbkViewResourceFile pbkRes( *env);
	pbkRes.OpenL();
	CleanupClosePushL(pbkRes);

	CPbkContactItem* aContactItem=0;
	TBool create_new = EFalse;
	//CPbkContactEngine * engine = aBook.get_engine();
	
	if (contact_id == -1) 
	{
		aContactItem=iEngine->CreateEmptyContactL();
		CleanupStack::PushL(aContactItem);
		create_new=ETrue;
	} 
	else 
	{
		aContactItem=iEngine->OpenContactLCX(contact_id);
		create_new=EFalse;
		if (duplicate) create_new=ETrue;
	}

	if (fieldInfo) 
	{
		TPbkContactItemField * field = aContactItem->AddOrReturnUnusedFieldL(*fieldInfo) ;
		if (field) field->TextStorage()->SetTextL(no);
		if (contact_id != -1)
		{
			focus_field_index = aContactItem->FindFieldIndex(*field);
		}
	}

	CPbkContactEditorDlg *ipPabDlg = CPbkContactEditorDlg::NewL(*iEngine, *aContactItem, create_new, focus_field_index );
	
	TInt res = KErrNone;
	CC_TRAPD( err, res = ipPabDlg->ExecuteLD());
	if (contact_id == -1) 
	{
		CleanupStack::PopAndDestroy(2);  //contact_item, pbkRes
	} 
	else 
	{
		CleanupStack::PopAndDestroy(3);  //contact_item, lock, pbkRes
	}
}

EXPORT_C void phonehelper_ui::send_as(TUid mtmUid, CArrayFixFlat<TInt> * c)
{
	CALLSTACKITEM_N(_CL("phonehelper_ui"), _CL("send_as"));

	CDesCArrayFlat* file_list = new CDesCArrayFlat(1);
	CleanupStack::PushL(file_list);
	
	// generate vcard files
	for (int i=0; i< c->Count(); i++)
	{
		TFileName filename = ExportVCardToFile(c->At(i));
		if (filename.Length() >0 ) file_list->AppendL(filename);
	}

	if (file_list->Count() <= 0) 
	{
		CleanupStack::PopAndDestroy();
		return;
	}
	
	if (mtmUid == KSenduiMtmSmsUid)  // sending as a BIO message
	{
		CParaFormatLayer* paraf=CParaFormatLayer::NewL();
		CleanupStack::PushL(paraf);

		CCharFormatLayer* charf=CCharFormatLayer::NewL();
		CleanupStack::PushL(charf);

		CRichText* body;
		body=CRichText::NewL(paraf, charf);
		CleanupStack::PushL(body);

		TInt last_char_idx = 0;
		
		for (int i=0; i<file_list->Count();i++)
		{
			RFile tempfile;
			TInt err = tempfile.Open(Fs(), (*file_list)[i], EFileRead);
			TInt size;
			err = tempfile.Size(size);
			HBufC8 * temp8 = HBufC8::NewL(size);
			CleanupStack::PushL(temp8);
			TPtr8 des=temp8->Des();
			tempfile.Read(des);
			tempfile.Close();
			// cleaning
			Fs().Delete((*file_list)[i]);
		
			HBufC * temp16 = HBufC::NewL(des.Length()+10);
			CleanupStack::PushL(temp16);
			temp16->Des().Copy(des);
			body->InsertL(last_char_idx, temp16->Des() );
			last_char_idx += temp16->Length();
			CleanupStack::PopAndDestroy(2);  //temp8, temp16
		}
		
		CDesCArrayFlat* a=new CDesCArrayFlat(1);
		CleanupStack::PushL(a);
					
                iSendAppUi->CreateAndSendMessageL(KSenduiMtmSmsUid , body, 0, KMsgBioUidVCard, a);
		CleanupStack::PopAndDestroy(4); //a, body, paraf, charf
	}
	else  // sending as attachment
	{
		CParaFormatLayer* paraf=CParaFormatLayer::NewL();
		CleanupStack::PushL(paraf);

		CCharFormatLayer* charf=CCharFormatLayer::NewL();
		CleanupStack::PushL(charf);

		CRichText* body;
		body=CRichText::NewL(paraf, charf);
		CleanupStack::PushL(body);

		CDesCArrayFlat* a=new CDesCArrayFlat(1);
		CleanupStack::PushL(a);			

		iSendAppUi->CreateAndSendMessageL(mtmUid , body, file_list, KNullUid, a);
                		
		CleanupStack::PopAndDestroy(4); //body, charf, paraf, a

		for (int i=0; i<file_list->Count();i++)
		{
			// cleaning
			Fs().Delete((*file_list)[i]);
		}
	}
	CleanupStack::PopAndDestroy(file_list);
}

EXPORT_C TFileName phonehelper_ui::ExportVCardToFile(TInt ContactId)
{
	CALLSTACKITEM_N(_CL("phonehelper_ui"), _CL("ExportVCardToFile"));

	TFileName filename;

	CBCardEngine* cardEngine = CBCardEngine::NewL(iEngine); 
	CleanupStack::PushL(cardEngine);

	CPbkContactItem* con = iEngine->ReadContactLC(ContactId);

	HBufC * title = con->GetContactTitleOrNullL();
	CleanupStack::PushL(title);
	if (title) {
		filename.Append(_L("C:\\"));
		filename.Append(*title);
		filename.Append(_L(".vcf"));
	} else {
		filename.AppendFormat(_L("C:\\unnamed%02d.vcf"), ContactId);
	}
	CleanupStack::PopAndDestroy(); //title

	RFile outfile;
	TInt err = outfile.Replace(Fs(), filename, EFileWrite);
	if (err != KErrNone) return _L("");
		
	RFileWriteStream writeStream(outfile);

	cardEngine->ExportBusinessCardL(writeStream, *con);
	CleanupStack::PopAndDestroy(con);

	writeStream.CommitL(); 
	writeStream.Close(); 
	outfile.Close(); 

	CleanupStack::PopAndDestroy(cardEngine);

	return filename;
}
