#include "break.h"
#include "audioengine.h"
#include "app_context.h"
#include <MdaAudioSamplePlayer.h>
#include "mplayeruicontrollerlistener.h"

class CAudioEngineImpl: public CAudioEngine, public MMdaAudioPlayerCallback
{
public:
	static CAudioEngineImpl* NewL();
	static CAudioEngineImpl* NewLC();
	~CAudioEngineImpl();

	void DoStopL();
	void DoPlayL();
	void SetNewFileL(const TDesC& aFileName);
	void DoPauseL();

	TInt IncreaseVolume();
	TInt DecreaseVolume();
   
	void MapcInitComplete(TInt aError, const TTimeIntervalMicroSeconds& aDuration);
	void MapcPlayComplete(TInt aError);

	void InitControllerL( MPlayerUIControllerListener* aCallback, TInt volume );
	void CloseController();
	TInt GetEngineState();
	enum TEngineState {EPNotInitialised, EPInitialising, EPStopped, EPPlaying, EPPaused};

private:
	CAudioEngineImpl();
	void ConstructL();

private:
	
	TEngineState iEngineState;
	MPlayerUIControllerListener* iListener;
	CMdaAudioPlayerUtility* iMdaPlayer;
	HBufC*	iMediaFile;
	TInt iVolume;

	friend CAudioEngine;
};

EXPORT_C CAudioEngine* CAudioEngine::NewL()
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("NewL"));

	CAudioEngine* self = NewLC();
	CleanupStack::Pop(self);
	return self;
}

    
EXPORT_C CAudioEngine* CAudioEngine::NewLC()
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("NewLC"));

	CAudioEngineImpl* self = new (ELeave) CAudioEngineImpl();
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
}

CAudioEngineImpl::CAudioEngineImpl()
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("CAudioEngineImpl"));

}


CAudioEngineImpl::~CAudioEngineImpl()
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("~CAudioEngineImpl"));

	delete iMediaFile;
	delete iMdaPlayer;    
	iMdaPlayer = NULL;
}


void CAudioEngineImpl::ConstructL()
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("ConstructL"));

	iMediaFile = HBufC::NewL(0);
	iEngineState = EPNotInitialised; 
}

void CAudioEngineImpl::SetNewFileL(const TDesC &aFileName)
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("SetNewFileL"));

	HBufC* newFile = aFileName.AllocL();
	delete iMediaFile;		 
	iMediaFile = newFile;
}

void CAudioEngineImpl::InitControllerL( MPlayerUIControllerListener* aCallback, TInt volume )   
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("InitControllerL"));

	iEngineState = EPInitialising;
	iListener = aCallback;
	iVolume = volume;
    
	delete iMdaPlayer;
	iMdaPlayer = NULL;
	{
		CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("InitControllerL1"));
		iMdaPlayer=CMdaAudioPlayerUtility::NewFilePlayerL(*iMediaFile,*this);
	}
	{
		CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("InitControllerL2"));
	}
}

void CAudioEngineImpl::DoPlayL()
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("DoPlayL"));

	iMdaPlayer->Play();
	iEngineState = EPPlaying;
}


void CAudioEngineImpl::DoStopL()
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("DoStopL"));

	iMdaPlayer->Stop();
	iEngineState = EPStopped;
	MapcPlayComplete(KErrNone);
}


void CAudioEngineImpl::MapcInitComplete(TInt aError, const TTimeIntervalMicroSeconds& /*aDuration*/)
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("MapcInitComplete"));

	if (aError == KErrNone) {
		iMdaPlayer->SetVolume(iVolume);
		if ( iListener ) {CC_TRAPD( ignore, iListener->InitControllerCompletedL( aError ) ); }
        } else {
		// acts as playing is over
		MapcPlayComplete(aError);
	}
}

void CAudioEngineImpl::DoPauseL()
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("DoPauseL"));

	if ( iEngineState == EPPlaying ) {
		iMdaPlayer->Stop();
		iEngineState = EPPaused;
	}
}

void CAudioEngineImpl::CloseController()
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("CloseController"));

	iListener=0;
	delete iMdaPlayer;
	iMdaPlayer = 0;
}

void CAudioEngineImpl::MapcPlayComplete(TInt aError)
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("MapcPlayComplete"));

	iListener->PlayCompletedL(aError);
	iEngineState = EPStopped;
}

TInt CAudioEngineImpl::GetEngineState()
{
	CALLSTACKITEM_N(_CL("CAudioEngineImpl"), _CL("GetEngineState"));

	return iEngineState;
}

TInt CAudioEngineImpl::IncreaseVolume()
{
	if ( iVolume< iMdaPlayer->MaxVolume()) {
		iMdaPlayer->SetVolume(++iVolume);
	}
	return iVolume;
}

TInt CAudioEngineImpl::DecreaseVolume()
{
	if ( iVolume> 0) {
		iMdaPlayer->SetVolume(--iVolume);
	}
	return iVolume;
}