#include "cm_taglist.h"
#include <avkon.hrh>
#include <aknselectionlist.h> 
#include <aknsfld.h>
#include <txtrich.h>
#include "symbian_auto_ptr.h"
#include <contextmediaui.rsg>
#include "cm_tags.h"
#include <aknmessagequerydialog.h>

enum KEYCODES {
	KEY_C = 0x0008
};

class CAknMarkableListDialogEx : public CAknMarkableListDialog {
public:
	CTagStorage*	iTagStorage;
	CAknMarkableListDialogEx(TInt &aValue, CArrayFix< TInt > *aSelectedItems, 
		CTagStorage *aArray, TInt aMenuBarResourceId, TInt aOkMenuBarResourceId, 
		MEikCommandObserver *aObserver) :
	CAknMarkableListDialog (aValue, aSelectedItems, aArray, aMenuBarResourceId,
		aOkMenuBarResourceId, aObserver), iTagStorage(aArray) { }
	static CAknMarkableListDialogEx* NewL( TInt &aOpenedItem, CArrayFix< TInt > *aSelectedItems, 
		CTagStorage *aArray, 
		TInt aMenuBarResourceId, MEikCommandObserver *aCommand = 0 ) {
		CAknMarkableListDialogEx* ret=new (ELeave) CAknMarkableListDialogEx(aOpenedItem, aSelectedItems,
			aArray, aMenuBarResourceId, 0, aCommand );
		ret->ConstructL(aMenuBarResourceId);
		return ret;
	}

	void DeleteItemL() {
		TInt idx=ListBox()->CurrentItemIndex();

		auto_ptr<HBufC> message(HBufC::NewL(256));
		message->Des()=_L("Really delete ");
		MListBoxModel* lbmodel=ListBox()->Model();
		CAknFilteredTextListBoxModel* model = STATIC_CAST( CAknFilteredTextListBoxModel*, lbmodel);

		TPtrC item=model->ItemText( idx );
		message->Des().Append( item.Mid(2) );
		message->Des().Append(_L("?"));

		{
			TPtrC16 m=message->Des();
			CAknMessageQueryDialog * dlg = CAknMessageQueryDialog::NewL(m);
			CleanupStack::PushL(dlg);
			dlg->PrepareLC(R_CONFIRMATION_QUERY);
			dlg->QueryHeading()->SetTextL(_L("Delete?"));
			CleanupStack::Pop(dlg);
			
			if ( dlg->RunLD() )
			{
				TInt realidx=model->Filter()->FilteredItemIndex(idx);
				iTagStorage->DeleteItemL(realidx);
				ListBox()->HandleItemRemovalL();
			}
		}	
	}

	virtual TBool OkToExitL  (  TInt  aButtonId   )  {
		if (aButtonId==-2) {
			const CArrayFix<TInt> *indexes = ListBox()->SelectionIndexes();
			if (indexes->Count()==0) {
				SelectionListProcessCommandL(EAknCmdMark);
			}
		}
		return CAknMarkableListDialog::OkToExitL(aButtonId);
	}
	virtual TKeyResponse OfferKeyEventL  (  const TKeyEvent &  aKeyEvent,  
		TEventCode  aType )
	{
		if (aType==EEventKey && aKeyEvent.iCode==0xF845) {
			TInt idx=ListBox()->CurrentItemIndex();
			if (idx!=-1) {
				TBool selected=EFalse;
				const CArrayFix<TInt> *indexes = ListBox()->SelectionIndexes();
				if (indexes) {
					for (int i=0; i<indexes->Count(); i++) {
						if (indexes->At(i)==idx) {
							selected=ETrue;
							break;
						}
					}
				}
				if (!selected) {
					SelectionListProcessCommandL(EAknCmdMark);
				} else {
					SelectionListProcessCommandL(EAknCmdUnmark);
				}
			}
			return EKeyWasConsumed;
		} else {
			if (aType==EEventKey && aKeyEvent.iCode==KEY_C && FindBox()->TextLength()==0) {
				DeleteItemL();
				return EKeyWasConsumed;
			} else {
				return CAknMarkableListDialog::OfferKeyEventL(aKeyEvent, aType);
			}
		}
	}
};

class CListSearch : public CActive {
public:
	CAknMarkableListDialogEx* dlg;
	CListSearch() : CActive(CActive::EPriorityStandard) { }
	void ConstructL() {
		CActiveScheduler::Add(this);
		TRequestStatus *s=&iStatus;
		User::RequestComplete(s, KErrNone);
		SetActive();
	}
	void DoCancel() { }
	TBuf<50> iSearchText;
	void RunL() {
		for (int i=0; i<iSearchText.Length(); i++) {
			TKeyEvent ev; 
			ev.iCode=iSearchText[i];
			TEventCode c(EEventKey);
			dlg->OfferKeyEventL(ev, c);
		}
	}
	~CListSearch() {
		Cancel();
	}
};

IMPORT_C TBool SelectTagsFromListL(class CTagStorage& aStorage, class CRichText* aText, TInt& aAddedFrom, 
				   TInt aSelectionStart, TInt aSelectionEnd)
{
	TPtrC text1=aText->Read(0);
	TPtrC text(0, 0);
	auto_ptr<HBufC> buf(0);
	if (aSelectionStart>=0 && aSelectionEnd>aSelectionStart) {
		buf.reset(HBufC::NewL(text1.Length()));
		buf->Des().Append(text1.Left(aSelectionStart));
		buf->Des().Append(text1.Mid(aSelectionEnd, text1.Length()-aSelectionEnd-1));
		text.Set(buf->Des());
	} else {
		text.Set(text1.Ptr(), text1.Length()-1); // ignore end-of-text character
	}

	TInt iSelectedItem=0;
	auto_ptr<CListSearch> s(new (ELeave) CListSearch);
	auto_ptr< CArrayFixFlat<TInt> > items(new (ELeave) CArrayFixFlat<TInt>(4));
	CAknMarkableListDialogEx  * dlg=CAknMarkableListDialogEx::NewL(iSelectedItem, items.get(),
		&aStorage, R_LIST_MENUBAR);
	dlg->PrepareLC(R_LIST_DIALOG);

	s->ConstructL(); s->dlg=dlg;
	TBool add_colon=ETrue;
	if (text.Length()==0 || text[text.Length()-1]==':' || text[text.Length()-1]==':') add_colon=EFalse;
	TInt colon_pos=KErrNotFound;
	/*
	if (colon_pos!=KErrNotFound && colon_pos!=text.Length()-1) {
		TInt len=text.Length()-colon_pos-1;
		if (len>50) len=50;
		s->iSearchText=text.Mid(colon_pos+1, len);
		s->iSearchText.Trim();
		if (aStorage.TagExists(s->iSearchText)) {
			add_colon=ETrue;
			s->iSearchText.Zero();
			colon_pos=KErrNotFound;
		}
	} else if (colon_pos==KErrNotFound) {
		TInt len=text.Length();
		if (len>50) len=50;
		s->iSearchText=text.Mid(0, len);
		s->iSearchText.Trim();
		if (aStorage.TagExists(s->iSearchText)) {
			add_colon=ETrue;
			s->iSearchText.Zero();
		}
	} */
	
	if (dlg->RunLD()) {
		if (items->Count()>0) {
			if (aSelectionStart>=0 && aSelectionEnd>aSelectionStart) {
				aText->DeleteL(aSelectionStart, aSelectionEnd-aSelectionStart);
			}
			if (colon_pos!=KErrNotFound && colon_pos!=text.Length()-1) {
				aText->DeleteL(colon_pos+1, text.Length()-colon_pos-1);
			}
			if (colon_pos==KErrNotFound && s->iSearchText.Length()>0) {
				aText->DeleteL(0, aText->DocumentLength());
			}
			aAddedFrom=aText->DocumentLength();
			for (int i=0; i<items->Count(); i++) {
				if (add_colon) {
					aText->InsertL(aText->DocumentLength(), _L(":"));
				}
				// removing "0\t" from the tag
				aText->InsertL(aText->DocumentLength(), 
					aStorage.MdcaPoint(items->At(i)).Mid(2));
				add_colon=ETrue;
			}
			aText->InsertL(aText->DocumentLength(), _L(":"));
			return ETrue;
		} else {
			return EFalse;
		}
	} else {
		return EFalse;
	}
}
