#ifndef __PYTHON_PTR_H__
#define __PYTHON_PTR_H__  

#include <e32std.h>
#include <e32base.h>
#include <Python.h>
#include <symbian_python_ext_util.h>

void ClosePythonIndirect(TAny* aPtr);

template<class Y> struct python_ptr_ref
{
  Y* iPtr;
  python_ptr_ref(Y* aPtr) : iPtr(aPtr) {}
};


template<class X> class python_ptr
{
public:  
  typedef X element_type;

  python_ptr(X* aPtr = 0): iPtr(aPtr), iBasePtr(aPtr)
  {
    CleanupStack::PushL(TCleanupItem(ClosePythonIndirect, (void*)&iBasePtr));
  }  
  
  python_ptr(python_ptr& aPtr): iPtr(aPtr.release()), iBasePtr(iPtr)
  {
    CleanupStack::PushL(TCleanupItem(ClosePythonIndirect, (void*)&iBasePtr));
  }

  python_ptr<X>& operator=(python_ptr<X>& aRhs)
  {
    if (&aRhs != this)
    {
      if (iBasePtr) iBasePtr->Release();
      iBasePtr = iPtr = aRhs.get();
      Py_XINCREF(iBasePtr);
    }
    return (*this); 
  }

  ~python_ptr() 
  { 
    CleanupStack::Pop();
    Py_XDECREF(iBasePtr);
  }

  X& operator *() const { return *iPtr; }
  X* operator ->() const { return iPtr; }
  bool operator !() const { return (iPtr==0); }
   
  X* get() const { return iPtr; }

  X* release()
 	{ 
    X* result = iPtr;
    iBasePtr = iPtr = 0;
		return result; 
  }

  void reset(X* aPtr = 0) {
    if (aPtr != iPtr) {
      Py_XDECREF(iBasePtr);
      iBasePtr = iPtr = aPtr;
    }
  }


  python_ptr(python_ptr_ref<X> aRef): iPtr(aRef.iPtr), iBasePtr(iPtr)
  {
    CleanupStack::PushL(TCleanupItem(ClosePythonIndirect, (void*)&iBasePtr));
  }


  template <class Y> operator python_ptr_ref<Y>() 
    { return python_ptr_ref<Y>(iPtr); }
  
private:
  X* iPtr;  
  PyObject* iBasePtr;
};

#endif
