#include <e32std.h>
#include <Python.h>
#include <symbian_python_ext_util.h>
#include "break.h"
#include "concretedata.h"

#include "bbpython.h"
#include "symbian_python_ptr.h"
#include "context_uids.rh"

#define MODULE_NAME contextphone
#define MODULE_NAME_STRING "contextphone"
#define MODULE_INIT_FUNC initcontextphone

PyObject* fnc_BBTimeToUnix(PyObject* self, PyObject* args);
PyObject* fnc_UnixToBBTime(PyObject* self, PyObject* args);

static const PyMethodDef ContextPhone_methods[] = {
	{ "bbsession", (PyCFunction)get_CPBBSession, METH_NOARGS, "gets the singleton Blackboard session" },
	{ "bbtime_to_unix", (PyCFunction)fnc_BBTimeToUnix, METH_VARARGS, "converts the contextphone string (UTC) time into unix time (seconds since epoch)" },
	{ "unix_to_bbtime", (PyCFunction)fnc_UnixToBBTime, METH_VARARGS, "converts a unix time into the contextphone string (UTC)" },
	{ 0, 0 }
};

PyObject* fnc_BBTimeToUnixL(PyObject* args)
{
	char* s=0; int len=0;

	if (!PyArg_ParseTuple(args, "u#", &s, &len)) return 0;
	TBBTime val(TTime(0), KNullDesC);
	val.FromStringL( TPtrC( (TUint16*)s, len ) );
	TDateTime epoch; epoch.Set(1970, EJanuary, 0, 0, 0, 0, 0);
	TTime e(epoch);
	TInt unixtime;
	TTimeIntervalSeconds secs;
	User::LeaveIfError(val().SecondsFrom(e, secs));
	unixtime=secs.Int();
	return PyLong_FromLong(unixtime);
}

PyObject* fnc_UnixToBBTimeL(PyObject* args)
{
	TInt unixtime=0;
	if (!PyArg_ParseTuple(args, "i", &unixtime)) return 0;
	TDateTime d; d.Set(1970, EJanuary, 0, 0, 0, 0, 0);
	TTime e(d);
	e+=TTimeIntervalSeconds(unixtime);
	TBBTime val(e, KNullDesC);
	TBuf<16> s; val.IntoStringL(s);
	return Py_BuildValue("u#", s.Ptr(), s.Length());
}

PyObject* fnc_BBTimeToUnix(PyObject* self, PyObject* args)
{
	PyObject* ret;
	CC_TRAPD(err, ret=fnc_BBTimeToUnixL(args));
	if (err!=KErrNone) return SPyErr_SetFromSymbianOSErr(err);
	return ret;
}
PyObject* fnc_UnixToBBTime(PyObject* self, PyObject* args)
{
	PyObject* ret;
	CC_TRAPD(err, ret=fnc_UnixToBBTimeL(args));
	if (err!=KErrNone) return SPyErr_SetFromSymbianOSErr(err);
	return ret;
}

#define METHOD_TABLE const_cast<PyMethodDef*>(&ContextPhone_methods[0])

#ifdef TESTBUILD
void ZeroGlobal();
#endif

struct TBBID {
	int		id;
	const char* const name;
};

static const TBBID Constants1[] = {
	{ 4, "BLUETOOTH_NEIGHBOURS" },
	{ 25, "OWN_BLUETOOTH" },
	{ 13, "PROFILE" },
	{ 24, "BASE" },
	{ 34, "LOCATION" },
	{ 40, "UNREAD_UNANSWERED" },
	{ 18, "USERACTIVITY" },
	{ 33, "USERGIVEN" },
	{ 31, "OWN_PRESENCE" },
	{ 15, "CHARGER", },
	{ 16, "NETWORK_COVERAGE" },
	{ 39, "CURRENT_CALENDAR_EVENTS" },
	{ 11, "CURRENT_APP" },
	{ 1, "CELLID" },
	{ 12, "GPS_NMEA" },
	{ 46, "LAST_GOOD_GPS_NMEA" },
	{ 23, "CELLNAME" },
	{ 42, "CITY" },
	{ 43, "COUNTRY" },
	{ 0, "ANYSENSOR" },
	{ 0, 0 }
};

DL_EXPORT(void) MODULE_INIT_FUNC()
{
#ifdef TESTBUILD
	ZeroGlobal();
#endif
	PyObject* module = 
		Py_InitModule(MODULE_NAME_STRING, METHOD_TABLE);
	if (!module) {
		return;
	}
	User::LeaveIfError(def_CPBBSession());
	PyObject* d=PyModule_GetDict(module);

	const TBBID* c=&Constants1[0];
	while (c->name) {
		TInt res=PyDict_SetItemString(d, c->name, 
			python_ptr<PyObject>(Py_BuildValue(
				"ii", CONTEXT_UID_CONTEXTSENSORS, c->id)
			).get()
		);
		c++;
	};
}

GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}
