#include "csd_calendar.h"

EXPORT_C const TTypeName& TBBCalendarEvent::Type() const
{
	return KCalendarEventType;
}

EXPORT_C TBool TBBCalendarEvent::Equals(const MBBData* aRhs) const
{
	const TBBCalendarEvent* rhs=bb_cast<TBBCalendarEvent>(aRhs);
	return (rhs && *this==*rhs);
}

EXPORT_C const TTypeName& TBBCalendarEvent::StaticType()
{
	return KCalendarEventType;
}

EXPORT_C const MBBData* TBBCalendarEvent::Part(TUint aPartNo) const
{
	switch(aPartNo) {
	case 0:
		return &iStartTime;
	case 1:
		return &iDescription;
	case 2:
		return &iEndTime;
	case 3:
		return &iEventId;
	default:
		return 0;
	}
}

_LIT(KDescription, "description");
_LIT(KStartTime, "start_time");
_LIT(KEndTime, "end_time");
_LIT(KEventId, "eventid");

EXPORT_C TBBCalendarEvent::TBBCalendarEvent(const TDesC& aName) : 
	TBBCompoundData(aName),
	iDescription(KDescription), iStartTime(KStartTime), iEndTime(KEndTime),
	iEventId(KEventId) { }

EXPORT_C bool TBBCalendarEvent::operator==(const TBBCalendarEvent& aRhs) const
{
	return (
		iDescription==aRhs.iDescription &&
		iStartTime()==aRhs.iStartTime() &&
		iEndTime()==aRhs.iEndTime() &&
		iEventId()==aRhs.iEventId() 
		);
}

_LIT(KSemicolon, ";");

EXPORT_C const TDesC& TBBCalendarEvent::StringSep(TUint aBeforePart) const
{
	if (aBeforePart>0 && aBeforePart<=3)
		return KSemicolon;
	else
		return KNullDesC;
}

EXPORT_C TBBCalendarEvent& TBBCalendarEvent::operator=(const TBBCalendarEvent& aRhs)
{
	iDescription()=aRhs.iDescription();
	iStartTime()=aRhs.iStartTime();
	iEndTime()=aRhs.iEndTime();
	iEventId()=aRhs.iEventId();

	return *this;
}

EXPORT_C MBBData* TBBCalendarEvent::CloneL(const TDesC& Name) const
{
	TBBCalendarEvent* ret=new (ELeave) TBBCalendarEvent(Name);
	*ret=*this;
	return ret;
}

EXPORT_C const TTypeName& TBBCalendar::Type() const
{
	return KCalendarType;
}

EXPORT_C TBool TBBCalendar::Equals(const MBBData* aRhs) const
{
	const TBBCalendar* rhs=bb_cast<TBBCalendar>(aRhs);
	return (rhs && *this==*rhs);
}

EXPORT_C const TTypeName& TBBCalendar::StaticType()
{
	return KCalendarType;
}

EXPORT_C const MBBData* TBBCalendar::Part(TUint aPartNo) const
{
	switch (aPartNo) {
	case 0:
		return &iPrevious;
	case 1:
		return &iCurrent;
	case 2:
		return &iNext;
	default:
		return 0;
	}
}

_LIT(KPrevious, "previous");
_LIT(KCurrent, "current");
_LIT(KNext, "next");

EXPORT_C TBBCalendar::TBBCalendar() : TBBCompoundData(KCalendar), 
	iPrevious(KPrevious), iCurrent(KCurrent), iNext(KNext), 
	iCreatedVersion(2), iUseVersion(2) { }

EXPORT_C TBBCalendar::TBBCalendar(TUint aVersion) : TBBCompoundData(KCalendar), 
	iPrevious(KPrevious), iCurrent(KCurrent), iNext(KNext), 
	iCreatedVersion(aVersion), iUseVersion(2) { }


EXPORT_C void TBBCalendar::InternalizeL(RReadStream& aStream)
{
	iUseVersion=iCreatedVersion;
	TBBCompoundData::InternalizeL(aStream);
	iUseVersion=2;
}

EXPORT_C bool TBBCalendar::operator==(const TBBCalendar& aRhs) const
{
	return (
		iPrevious==aRhs.iPrevious &&
		iCurrent==aRhs.iCurrent &&
		iNext == aRhs.iNext
		);
}

_LIT(KHash, "#");

EXPORT_C const TDesC& TBBCalendar::StringSep(TUint aBeforePart) const
{
	if (aBeforePart>0 && aBeforePart<=2)
		return KHash;
	else
		return KNullDesC;
}

EXPORT_C TBBCalendar& TBBCalendar::operator=(const TBBCalendar& aRhs)
{
	iPrevious=aRhs.iPrevious;
	iCurrent=aRhs.iCurrent;
	iNext = aRhs.iNext;

	return *this;
}

EXPORT_C MBBData* TBBCalendar::CloneL(const TDesC& ) const
{
	TBBCalendar* ret=new (ELeave) TBBCalendar;
	*ret=*this;
	return ret;
}