#include "csd_event.h"

EXPORT_C const TTypeName& CBBSensorEvent::Type() const
{
	return KEventType;
}

EXPORT_C TBool CBBSensorEvent::Equals(const MBBData* aRhs) const
{
	const CBBSensorEvent* rhs=bb_cast<CBBSensorEvent>(aRhs);
	return (rhs && *rhs==*this);
}

EXPORT_C const TTypeName& CBBSensorEvent::StaticType()
{
	return KEventType;
}

EXPORT_C const MBBData* CBBSensorEvent::Part(TUint aPartNo) const
{
	if (aPartNo==0) return &iStamp;
	if (aPartNo==1) return &iPriority;
	if (aPartNo==2) return &iName;
	if (aPartNo==3) return &iData;
	return 0;
}

_LIT(KEventName, "eventname");
_LIT(KEventValue, "eventdata");

EXPORT_C CBBSensorEvent::CBBSensorEvent(const TDesC& aName, const TTupleName& aTupleName,
					MBBDataFactory* aFactory, TTime aTime, MBBData* aData, TPriority aPriority) : 
	TBBCompoundData(aName), iStamp(aTime, KStamp), iPriority(aPriority, KPriority), iName(aName, KEventName),
		iData(KEventValue, aFactory, aData), iFactory(aFactory), iTupleName(aTupleName)
{
}

EXPORT_C const TDesC& CBBSensorEvent::Name() const
{
	return iName();
}

EXPORT_C MBBData* CBBSensorEvent::GetPart(const TDesC& aName, const TTypeName& aType, TUint& aPartNoInto)
{
	MBBData* p=TBBCompoundData::GetPart(aName, aType, aPartNoInto);
	if (!p) {
		iData.SetValue(iFactory->CreateBBDataL(aType, aName, iFactory));
		aPartNoInto=3;
		p=&iData;
	}
	return p;
}

EXPORT_C bool CBBSensorEvent::operator==(const CBBSensorEvent& aRhs) const
{
	return (
		iStamp == aRhs.iStamp &&
		iPriority == aRhs.iPriority &&
		iData.Equals(aRhs.iData()) &&
		iName == aRhs.iName );
}

_LIT(KSpace, " ");

EXPORT_C const TDesC& CBBSensorEvent::StringSep(TUint aBeforePart) const
{
	if (aBeforePart==0 || aBeforePart > 2) return KNullDesC;
	return KSpace;
}

EXPORT_C const TTupleName& CBBSensorEvent::TupleName() const
{
	return iTupleName;
}

EXPORT_C CBBSensorEvent& CBBSensorEvent::operator=(const CBBSensorEvent& aSensorEvent)
{
	MBBData* val=0;
	if (aSensorEvent.iData()) val=aSensorEvent.iData()->CloneL(aSensorEvent.iData()->Name());

	iStamp()=aSensorEvent.iStamp();
	iPriority()=aSensorEvent.iPriority();
	iData.SetValue(val);
	iData.SetOwnsValue(ETrue);
	iName()=aSensorEvent.iName();
	return *this;
}

EXPORT_C MBBData* CBBSensorEvent::CloneL(const TDesC& Name) const
{
	auto_ptr<CBBSensorEvent> ret(new (ELeave) CBBSensorEvent(Name, iTupleName, iFactory));
	*ret=*this;
	return ret.release();
}
