#include "csd_gps.h"

EXPORT_C const TTypeName& TGpsLine::Type() const
{
	return KGpsLineType;
}

EXPORT_C const TTypeName& TGpsLine::StaticType()
{
	return KGpsLineType;
}

EXPORT_C TBool TGpsLine::Equals(const MBBData* aRhs) const
{
	const TGpsLine* rhs=bb_cast<TGpsLine>(aRhs);
	return (rhs && *this==*rhs);
}


EXPORT_C TGpsLine& TGpsLine::operator=(const TGpsLine& aGpsLine)
{
	Value()=aGpsLine.Value();
	return *this;
}

EXPORT_C MBBData* TGpsLine::CloneL(const TDesC&) const
{
	TGpsLine* ret=new (ELeave) TGpsLine();
	*ret=*this;
	return ret;
}

void DegAndMinutesToGeo(const TDesC& aMin, TDes& aInto, TInt aSign)
{
	aInto.Zero();
	if (aMin.Compare(_L("0000.0000"))==0  || aMin.Compare(_L("00000.0000"))==0  || aMin.Length()<3) {
		return;
	}
	TInt decpos=aMin.Locate('.');
	if (decpos==KErrNotFound || decpos<3) return;

	TReal deg, min;
	{
		TLex lex(aMin.Left(decpos-2));
		if (lex.Val(deg)!=KErrNone) {
			return;
		}
	}

	{
		TLex lex(aMin.Mid(decpos-2));
		if (lex.Val(min)!=KErrNone) {
			return;
		}
		if (deg==TReal(0) && min==TReal(0)) return;
		min/=TReal(60);
	}

	deg+=min;

	if (deg > TReal(180)) return;

	deg*=aSign;
	TRealFormat fmt; fmt.iTriLen=0;
	fmt.iPoint='.'; fmt.iPlaces=8;
	fmt.iType=KRealFormatCalculator;
	aInto.Num(deg, fmt);
}

EXPORT_C void NmeaToLatLong(const TDesC& aNmea, TGeoLatLong& aInto)
{
	aInto.iLat.Zero();
	aInto.iLong.Zero();
	// invalid $GPRMC,071546.557,V,6010.6681,N,02457.6963,E,,,070205,,*12
	// valid $GPRMC,143041.940,A,6009.7988,N,02454.3641,E,0.00,,010206,,*10
	TLex lex(aNmea);
	TInt sign=1;
	TBuf<20>  tmp;
	int i=0;
	for (i=0; i<3; i++) {
		// skip $GPRMC,071546.557,V,
		while (! lex.Eos() && lex.Peek() != ',') lex.Inc();
		if (lex.Eos()) goto no_data;
		lex.Inc();
	}
	lex.Mark();
	while (! lex.Eos() && lex.Peek() != ',') lex.Inc();
	if (lex.Eos()) goto no_data;

	tmp=lex.MarkedToken(); lex.Inc();
	if (lex.Get()=='S') {
		sign=-1;
	}
	DegAndMinutesToGeo(tmp, aInto.iLat, sign);

	lex.Inc();
	lex.Mark();
	while (! lex.Eos() && lex.Peek() != ',') lex.Inc();
	if (lex.Eos()) goto no_data;

	tmp=lex.MarkedToken(); lex.Inc();
	sign=1;
	if (lex.Get()=='W') {
		sign=-1;
	}
	DegAndMinutesToGeo(tmp, aInto.iLong, sign);

	// elevation/lock flag

	lex.Inc();
	lex.Mark();
	while (! lex.Eos() && lex.Peek() != ',') lex.Inc();
	if (lex.Eos()) 
		goto no_data;
	if (lex.MarkedToken().Length()==0) 
		goto no_data;


	return;

no_data:
	aInto.iLat.Zero();
	aInto.iLong.Zero();
	return;
}
