/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/

/*
 * FIXME: we should use active objects and the active scheduler. The
 * code is a mess.
 */

#include <e32base.h>
#include <e32cons.h>
#include <stdlib.h>
#include <apgcli.h>
#include <apgtask.h>
#include <w32std.h>
#include <saclient.h>

#include "context_uids.h"
#include <bautils.h>
#ifdef __S60V2__
#include <sysutil.h>
#endif
#include <APACMDLN.H>
#include <eikdll.h>
#include <flogger.h>

void DebugMsg(const TDesC& aMessage, RFileLogger& aDebugLog) {
	aDebugLog.Write(aMessage);
}

void KillApplicationL(RWsSession& aWs, TUid aUid, RFileLogger& aDebugLog)
{
	TTime wait_until; wait_until.HomeTime();
	wait_until+=TTimeIntervalSeconds(15);
	
	TApaTaskList taskList( aWs );
	{
		TApaTask task = taskList.FindApp(aUid);

		if( task.Exists() ) {
			DebugMsg(_L("sending EApaSystemEventShutdown"), aDebugLog);
			task.SendSystemEvent(EApaSystemEventShutdown);
			DebugMsg(_L("sent EApaSystemEventShutdown"), aDebugLog);
		}
	}

	for(;;) {
		TApaTask task = taskList.FindApp(aUid);
		if(! task.Exists()) {
			break;
		}
		TTime now; now.HomeTime();
		if (now < wait_until) {
			DebugMsg(_L("waiting..."), aDebugLog);
			User::After(TTimeIntervalMicroSeconds32(500*1000) );
		} else {
			break;
		}
	}
	TApaTask task = taskList.FindApp(aUid);
	if( task.Exists()) {
		RThread t;
		if (t.Open(task.ThreadId())==KErrNone) {
			DebugMsg(_L("killing"), aDebugLog);
			t.Kill(2003);
			t.Close();
		}
	}
}

void KillExeL(const TDesC& aProcessName, RFileLogger& aDebugLog)
{
	DebugMsg(_L("finding processes"), aDebugLog);
	TFindProcess f(_L("*"));
	TFullName t;
	RProcess r;
	TInt len=aProcessName.Length();
	if (len==0) User::Leave(KErrArgument);
	while (f.Next(t)==KErrNone) {
		r.Open(t);
		CleanupClosePushL(r);
		if (r.FileName().Length()>=len && 
			r.FileName().Left(1).CompareF(_L("z"))!=0 && 
			r.FileName().Mid(1).CompareF(aProcessName.Mid(1))==0) {
			DebugMsg(_L("killing"), aDebugLog);
			r.Kill(2003);
		}
		CleanupStack::PopAndDestroy();
	}
}

// This is the mainloop game loop or whatever
void mainloop(TDes8& aState, RFileLogger& aDebugLog)
{
	TInt pushed=0;
	RWsSession ws; 
	User::LeaveIfError(ws.Connect());
	CleanupClosePushL(ws); pushed++;

#ifndef __WINS__
	DebugMsg(_L("KillExeL starter"), aDebugLog);
	KillExeL(_L("\\system\\apps\\starter\\starter.exe"), aDebugLog);
#else
	DebugMsg(_L("KillApplication starter"), aDebugLog);
	KillApplicationL(ws, KUidstarter, aDebugLog);
#endif
	DebugMsg(_L("KillApplication context_log"), aDebugLog);
	KillApplicationL(ws, KUidcontext_log, aDebugLog);
	DebugMsg(_L("KillApplication context_log"), aDebugLog);
	KillApplicationL(ws, KUidContextContacts, aDebugLog);
	DebugMsg(_L("KillApplication context_log"), aDebugLog);
	KillApplicationL(ws, KUidContextCallLog, aDebugLog);

	CleanupStack::PopAndDestroy(pushed);

}

_LIT(KTxtEPOC32EX,"SHUTTER");


void reporterror(TInt error, const TDesC8& state)
{
	RFs fs; if (fs.Connect()!=KErrNone) return;
	RFile f; 
	if (f.Replace(fs, _L("c:\\system\\data\\context\\shutter_error.txt"), EFileWrite|EFileShareAny)
		==KErrNone) {

		f.Write(_L8("Error in shutter: "));
		TBuf8<12> b; b.AppendNum(error); b.Append(_L8("\n"));
		f.Write(b);
		f.Write(state);
		f.Close();
	}
	fs.Close();
}

#ifdef __WINS__
EXPORT_C TInt InitEmulator() // mainloop function called by the emulator software
#else
GLDEF_C TInt E32Main() // mainloop function called by E32
#endif
{
	__UHEAP_MARK;
	CTrapCleanup* cleanup=CTrapCleanup::New(); // get clean-up stack
	TBuf8<80> state;
	RFileLogger log;
	TInt error=KErrNone;
	state=_L8("creating logger");
	if ((error=log.Connect())==KErrNone) {
		log.CreateLog(_L("shutter"), _L("log.txt"), EFileLoggingModeAppend);
		TRAP(error,mainloop(state, log)); // more initialization, then do example
		DebugMsg(_L("finished"), log);
		log.Close();
	}
	state=_L8("shutting downs");
	if (error!=KErrNone) reporterror(error, state);
	__ASSERT_ALWAYS(!error,User::Panic(KTxtEPOC32EX,error));
	delete cleanup; // destroy clean-up stack
	__UHEAP_MARKEND;
	
	//CloseSTDLIB();
	User::Exit(0);
	
	return KErrNone;
}

#ifdef __WINS__

int GLDEF_C E32Dll(TDllReason)
{
	return(KErrNone);
}

#endif
