#include "icons.h"

#include <eikenv.h> 
#include <contextcommon.mbg>
#include <bautils.h>
#include <s32file.h>
#include <gulicon.h>
#include "app_context.h"

#include <symbian_auto_ptr.h>

EXPORT_C TInt GetIconBitmap(TInt index, const TIconID* aIconDefs, TInt aNbIcons)
{
	return aIconDefs[index].iBitmap;
}

EXPORT_C TInt GetIconMask(TInt index, const TIconID* aIconDefs, TInt aNbIcons)
{
	return aIconDefs[index].iMask;
}

EXPORT_C TPtrC GetIconMbm(TInt index, const TIconID* aIconDefs, TInt aNbIcons)
{
	return TPtrC((TText16*)aIconDefs[index].iMbmFile);
}

void GetIconInfo(const TDesC& FileName, RFs& Fs, RFile& File, RFileReadStream& s, RArray<TInt>& Offsets, int round)
{
	TBuf<50> c=_L("GetIconInfo round "); c.AppendNum(round);

	TInt table_offset=0;
	{

		User::LeaveIfError(File.Open(Fs, FileName, EFileRead|EFileShareAny));
		RFile f0=File;
		s.Attach(f0, 0); 
		TInt uid=s.ReadUint32L();
		TUid file_uid={ uid };
		if (file_uid==KMultiBitmapRomImageUid) {
			table_offset=0x4;
		} else {
			uid=s.ReadUint32L(); // uid2
			uid=s.ReadUint32L(); // uid3
			uid=s.ReadUint32L(); // uid4 == checksum
			table_offset=s.ReadUint32L();
		}
	}

	TInt size=0; 
	{
		c=_L("GetIconInfo::2::"); c.AppendNum(table_offset);
		RFile f=File;
		s.Attach(f, table_offset);
		size=s.ReadUint32L();
		Offsets.Reset();
	}

/*
	MApp_context* ctx=GetContext();
	c=_L("offset::"); c.AppendNum(table_offset); c.Append(_L(" ")); 
	c.Append(_L("size::")); c.AppendNum(size); c.Append(_L(" i::0"));
*/
	int i=0;
	for (i=0; i<size; i++) {
		TInt offset=s.ReadUint32L();
		User::LeaveIfError(Offsets.Append(offset));
/*
		c=_L("offset::"); c.AppendNum(table_offset); c.Append(_L(" ")); 
		c.Append(_L("size::")); c.AppendNum(size); c.Append(_L(" i::")); c.AppendNum(i);
*/
	}
}

EXPORT_C void LoadIcons(CArrayPtrFlat<CGulIcon> * aIconList, const TIconID* aIconDefs, 
			TInt aNbIcons)
{
	LoadIcons(aIconList, aIconDefs, aNbIcons, 1);
}

EXPORT_C void ScaleFbsBitmapL(CFbsBitmap * orig, CFbsBitmap * dest, TInt aScale)
{
	TSize s = orig->SizeInPixels();
	TInt width=s.iWidth;
	TInt height=s.iHeight;
	s.iWidth*=aScale;
	s.iHeight*=aScale;
	User::LeaveIfError(dest->Create(s, orig->DisplayMode()));

	TSize twips = orig->SizeInTwips();
	twips.iWidth*=aScale;
	twips.iHeight*=aScale;
	dest->SetSizeInTwips(twips);

	TBitmapUtil orig_util(orig);
	TBitmapUtil dest_util(dest);
	orig_util.Begin(TPoint(0,0)); 
	dest_util.Begin(TPoint(0,0), orig_util); 

	TInt xPos;
	for (TInt yPos=0;yPos<height;yPos++) {
		for (TInt j=0; j<aScale; j++) {
			orig_util.SetPos(TPoint(0,yPos));
			dest_util.SetPos(TPoint(0,yPos*aScale+j));
			for (xPos=0;xPos<width;xPos++) {
				for (TInt k=0; k<aScale; k++) {
					dest_util.SetPixel(orig_util);
					dest_util.IncXPos();
				}
				orig_util.IncXPos();
			}
		}
	}
	orig_util.End();
	dest_util.End();
}

EXPORT_C void LoadIcons(CArrayPtrFlat<CGulIcon> * aIconList, const TIconID* aIconDefs, 
			TInt aNbIcons, TInt aScale)
{
	CALLSTACKITEM_N(_CL("User"), _CL("LeaveIfError"));

	TFileName real; TFileName prev;
	CEikonEnv* env=CEikonEnv::Static();
	RFs& fs=env->FsSession();
	CWsScreenDevice* screen=env->ScreenDevice();
	RWsSession& ws=env->WsSession();
	RArray<TInt> Offsets; RFile File; bool file_is_open=false;
	CleanupClosePushL(Offsets);
	RFileReadStream s;
	int j=0;

	bool romfile=false;
	for (int i = 0; i<aNbIcons;i++)
	{
		TPtrC file((TText16*)aIconDefs[i].iMbmFile);
		if (prev.Compare(file)) {
			real=file;
#ifdef __WINS__
			real.Replace(0, 1, _L("z"));
#else
			if (! BaflUtils::FileExists(fs, real)) {
				real.Replace(0, 1, _L("e"));
			}
#endif
			prev=file;
			if (file_is_open) {
				s.Close();
				file_is_open=false;
			}
			if (real.Left(1).CompareF(_L("z"))==0) {
				romfile=true;
			} else {
				romfile=false;
				GetIconInfo(real, fs, File, s, Offsets, j);
				file_is_open=true;
			}
			++j;
		}

		auto_ptr<CWsBitmap> bitmap(new (ELeave) CWsBitmap(ws));
		if (!romfile) {
			RFile f1=File;
			s.Attach(f1, Offsets[aIconDefs[i].iBitmap]);	
			bitmap->InternalizeL(s);
		} else {
			User::LeaveIfError(bitmap->Load(real, aIconDefs[i].iBitmap));
		}
		bitmap->SetSizeInTwips(screen);
#ifdef __S60V2__
		if (aScale>1) {
			auto_ptr<CWsBitmap> scaled(new (ELeave) CWsBitmap(ws));
			ScaleFbsBitmapL(bitmap.get(), scaled.get(), aScale);
			bitmap=scaled;
		}
#endif

		auto_ptr<CWsBitmap> mask(new (ELeave) CWsBitmap(ws));
		if (!romfile) {
			RFile f2=File;
			s.Attach(f2, Offsets[aIconDefs[i].iMask]);
			mask->InternalizeL(s);
		} else {
			User::LeaveIfError(mask->Load(real, aIconDefs[i].iMask));
		}
		mask->SetSizeInTwips(screen);
#ifdef __S60V2__
		if (aScale>1) {
			auto_ptr<CWsBitmap> scaled(new (ELeave) CWsBitmap(ws));
			ScaleFbsBitmapL(mask.get(), scaled.get(), aScale);
			mask=scaled;
		}
#endif

		auto_ptr<CGulIcon> icon(CGulIcon::NewL(bitmap.get(), mask.get()));
		bitmap.release(); mask.release();
		aIconList->AppendL(icon.get()); icon.release();
	}
	if (file_is_open) {
		s.Close();
	}
	CleanupStack::PopAndDestroy(); // Offsets
}

EXPORT_C TInt GetIconIndex(TInt identifier, const TIconID* aIconDefs, TInt aNbIcons)
{
	for (int i = 0; i<aNbIcons;i++)
	{
		if ( aIconDefs[i].iBitmap == identifier )
		{
			return i;
		}
	}	
	return 0; // index of empty icon
}
