/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


#pragma warning(disable: 4706)

// INCLUDE FILES
#include "ContextLocaAppUi.h"
#include <contextloca.rsg>
#include <sendnorm.rsg>
#include "local_defaults.h"
#include "context_log.hrh"
#include "contextlocaapp.h"

#define CONTEXTNW 1

#include <avkon.hrh>
#include <aknquerydialog.h> 
#include <eikenv.h> // ieikonenv

#include <eikmenup.h>
#include <apgcli.h>

#include <bautils.h>

#include <apgwgnam.h>

#include "cl_settings.h"

#include "break.h"

#include "notifystate.h"
#include <contextloca.mbg>
#include "viewids.h"

#include "raii_apgcli.h"
#include "raii_f32file.h"

#include "loca_sender.h"
#include "sms_status.h"
#include "callstack.h"
#include "reporting.h"
#include "app_context_impl.h"

#include "contextnotifyclientsession.h"

#include "bbl_runner.h"

#include "cu_errorui.h"
#include "emptytext.h"
#include "file_logger.h"
#include "csd_battery.h"
#include "csd_system.h"
#include "transfer.h"
#include "context_logcontainer.h"
#include "call_recorder.h"
#include <eikdll.h>
#include "mediaprompt.h"
#include "ntpconnection.h"
#include "cellnaming.h"
#include "settingsview.h"
#include "sms_snapshot.h"

_LIT(cellid_file, "cellid_names_v3.txt");
_LIT(transfer_cellid_file, "cellid_names_trans_v3.txt");

//#include <syevdef.h>

enum JOYSTICK_EVENTS {
	JOY_LEFT = 0xF807,
	JOY_RIGHT = 0xF808,
	JOY_UP = 0xF809,
	JOY_DOWN = 0xF80A,
	JOY_CLICK = 0xF845
};

const TUid KUidPhone = { 0x100058b3 };
const TUid KUidMenu = {  0x101f4cd2 };

#ifndef __WINS__
_LIT(KIconFile, "c:\\system\\data\\contextloca.mbm");
#else
_LIT(KIconFile, "z:\\system\\data\\contextloca.mbm");
#endif

#ifdef __WINS__
class TDummyMsvObserver : public MMsvSessionObserver
{
	void HandleSessionEvent(TMsvSessionEvent, TAny*, TAny*, TAny*) { }
	void HandleSessionEventL(TMsvSessionEvent, TAny*, TAny*, TAny*) { }
};
#endif


void CContextLocaAppUi::ConstructL()
{
	CALLSTACKITEM_N(_CL("CContextLocaAppUi"), _CL("ConstructL"));

#ifndef __WINS__
	bool wins=false;
#else
	bool wins=true;
#endif

	CContextLogAppUiBase::ConstructL();

#ifndef DONT_LOG_EVENTS_TO_FILE

	iLog->SubscribeL(KCellIdTuple);
	iLog->SubscribeL(KBluetoothTuple);
	iLog->SubscribeL(KOwnBluetoothTuple);
	iLog->SubscribeL(KBatteryTuple);
	iLog->SubscribeL(KNetworkTuple);
	iLog->SubscribeL(KChargerTuple);
	iLog->SubscribeL(KAppEventTuple);

#else
	// we always want errors to be logged
	iLog->SubscribeL(KAppEventTuple);
#endif

	state=_L("bbl");
	bbl=CBBLogger::NewL(AppContext(),this);

	{
		TBool logging_enabled;
		Settings().GetSettingL(SETTING_LOGGING_ENABLE, logging_enabled);
		TInt notif_err;
		CC_TRAP(notif_err, iLoggingRunning=CNotifyState::NewL(AppContext(), KIconFile));
		if (logging_enabled) {
			if(iLoggingRunning) iLoggingRunning->SetCurrentState( EMbmContextlocaL, EMbmContextlocaL );
		} else {
			if(iLoggingRunning) iLoggingRunning->SetCurrentState( EMbmContextlocaL_not, EMbmContextlocaL_not );
		}
	}

	state=_L("LocaSender");
	iLocaSender=CLocaSender::NewL(AppContext());

	state=_L("notify");

	Settings().NotifyOnChange(SETTING_LOGGING_ENABLE, this);

	state=_L("sms status");
	if (smsh) iSmsStatus=CSmsStatusReplier::NewL(AppContext(), smsh);

	ConstructAfterPresenceMaintainerL();

	if (wins) 
		iAppContainer->SubscribeL(KIdleTuple);

	state=_L("create transferer");
	transferer=CSendUITransfer::NewL(AppContext(), this, 
		Econtext_logCmdSendAppUi, DataDir(), AppDir());
	transferer->add_filesL(_L("log*txt"), true);
	transferer->add_filesL(_L("starter*txt"), true);
	transferer->add_filesL(_L("book*txt"), true);
	transferer->add_filesL(_L("calllog*txt"), true);
	transferer->add_filesL(_L("comm*txt"), false);
	transferer->add_filesL(_L("rec*amr"), false);
	transferer->add_filesL(_L("cellid_names_trans.txt"), false);
	transferer->add_filesL(_L("mms*.*"), false);

	state=_L("create settings view");

	{
		auto_ptr<CSettingsView> iSettingsView(CSettingsView::NewL(KSettingsViewId, AppContext(), 
			TLocalAppSettings::GetEnabledSettingsArray()));
		AddViewL(iSettingsView.get());
		iSettingsView.release();
	}

	FinalConstructL();

}

// ----------------------------------------------------
// CContextLocaAppUi::~CContextLocaAppUi()
// Destructor
// Frees reserved resources
// ----------------------------------------------------
//
CContextLocaAppUi::~CContextLocaAppUi()
{
	CALLSTACKITEM_N(_CL("CContextLocaAppUi"), _CL("~CContextLocaAppUi"));
	TInt err;

	{
		CLocalNotifyWindow::Destroy();

		TTime t2;
		t2=GetTime();
		if (iLog) 
			CC_TRAP(err, iLog->new_value(CBBSensorEvent::VALUE, _L("APP close"), state, t2));


		Settings().CancelNotifyOnChange(SETTING_LOGGING_ENABLE, this);
		Settings().CancelNotifyOnChange(SETTING_OWN_DESCRIPTION, this);

	}


	{
		TTime t; t=GetTime();
		if (iLog) CC_TRAP(err,
			iLog->new_value(CBBSensorEvent::VALUE, _L("APP"), _L("3"), t));
		delete transferer;
	}


	{
		CALLSTACKITEM_N(_CL("CContextLocaAppUi"), _CL("~CContextLocaAppUi_d1"));
		delete iSmsStatus;
	}
	{
		CALLSTACKITEM_N(_CL("CContextLocaAppUi"), _CL("~CContextLocaAppUi_d2"));
		delete iBBLocal;
	}
	{
		CALLSTACKITEM_N(_CL("CContextLocaAppUi"), _CL("~CContextLocaAppUi_d4"));
		delete bbl;
	}

	{
		CALLSTACKITEM_N(_CL("CContextLocaAppUi"), _CL("~CContextLocaAppUi_d6"));
		delete iContextRunning;
	}
	{
		CALLSTACKITEM_N(_CL("CContextLocaAppUi"), _CL("~CContextLocaAppUi_d5"));
		delete iLoggingRunning;
	}

	{
		CALLSTACKITEM_N(_CL("CContextLocaAppUi"), _CL("~CContextLocaAppUi_d7"));
		// near the end, since the cancelling doesn't work
		// right on s60v1 :-(
		delete iLocaSender;
	}

}


// ------------------------------------------------------------------------------
// CContextLocaAppUi::::DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane)
//  This function is called by the EIKON framework just before it displays
//  a menu pane. Its default implementation is empty, and by overriding it,
//  the application can set the state of menu items dynamically according
//  to the state of application data.
// ------------------------------------------------------------------------------
//
void CContextLocaAppUi::DynInitMenuPaneL(
					 TInt aResourceId,CEikMenuPane* aMenuPane)
{
	CALLSTACKITEM_N(_CL("CContextLocaAppUi"), _CL("DynInitMenuPaneL"));

	switch(aResourceId) {
	case R_OTHERS_MENU:
#ifndef REALLY_ONLY_LOGGING
		if (iLog) {
			if (iLog->is_paused()) {
				aMenuPane->SetItemDimmed(Econtext_logCmdAppPauseLog, ETrue);	
				aMenuPane->SetItemDimmed(Econtext_logCmdAppUnPauseLog, EFalse);
			} else {
				aMenuPane->SetItemDimmed(Econtext_logCmdAppPauseLog, EFalse);	
				aMenuPane->SetItemDimmed(Econtext_logCmdAppUnPauseLog, ETrue);
			}
		} else {
			aMenuPane->SetItemDimmed(Econtext_logCmdAppPauseLog, ETrue);	
			aMenuPane->SetItemDimmed(Econtext_logCmdAppUnPauseLog, ETrue);
		}
#endif
		break;

	case R_USER_ACTIONS:

		aMenuPane->SetItemDimmed(Econtext_logCmdAppSuspendPresence, ETrue);	
		aMenuPane->SetItemDimmed(Econtext_logCmdAppResumePresence, ETrue);
		break;
	case R_SENDUI_TOPMENU:
		transferer->DisplaySendMenuL(*aMenuPane, 1);
		break;
	case R_SENDUI_MENU:
		// sendui cascade
		transferer->DisplayMenuL(*aMenuPane);
		break;
	default:
		break;
	}
}



void CContextLocaAppUi::HandleCommandL(TInt aCommand)
{
	CALLSTACKITEM_N(_CL("CContextLocaAppUi"), _CL("HandleCommandL"));

	SetInHandlableEvent(ETrue);
#ifdef __WINS__
	TInt err;
	TBreakItem b(GetContext(), err);
#endif
	if (BaseHandleCommandL(aCommand)) return;

	switch ( aCommand )
	{
	case Econtext_logCmdAppPauseLog:
		Settings().WriteSettingL(SETTING_LOGGING_ENABLE, EFalse);
		break;
	case Econtext_logCmdAppUnPauseLog:
		Settings().WriteSettingL(SETTING_LOGGING_ENABLE, ETrue);
		break;
	case Econtext_logCmdCancelSend:
		// do nothing
		break;
	case Econtext_logCmdAppSettings:
		ActivateLocalViewL(KSettingsViewId);
		break;
	case Econtext_logCmdAppTest:
		{
		//loc->test();
		//recorder->test();
		DialogTest();
		}
		break;
	case Econtext_logCmdAppImsi:
		{
			/*if (loc) status_change(loc->GetImsi());
			else */{
#ifndef __WINS__
				TPlpVariantMachineId machineId;
				PlpVariant::GetMachineIdL(machineId);
				status_change(machineId);
#else
				// Return a fake IMEI when working on emulator
				_LIT(KEmulatorImsi, "244050000000000");
				status_change(KEmulatorImsi);
#endif
			}
		}
			
		break;
	default:
		if (aCommand>Econtext_logCmdSendAppUi || aCommand==Econtext_logCmdSendFtp) {
			
			if (aCommand==Econtext_logCmdSendFtp) {
				//iPeriodicTransfer->Transfer(false);
				//iPeriodicTransfer->Transfer(true);
			} else {
				status_change(_L("trying transfer"));

				/*
				cellid_name_file.Close();
				*/
				
				TFileName transfer_cellid_filen, cellid_filen;
				transfer_cellid_filen.Format(_L("%S%S"), &AppDir(), &transfer_cellid_file);
				cellid_filen.Format(_L("%S%S"), &DataDir(), &cellid_file);

				TInt ferr=BaflUtils::CopyFile(Fs(), cellid_filen, transfer_cellid_filen);
				if (ferr!=KErrNone) {
					TBuf<30> errmsg;
					errmsg.Format(_L("copy: %d"), ferr);
					error(errmsg);
					return;
				}
				/*
				ferr=cellid_name_file.Open(Fs(), cellid_filen, 
					EFileShareAny | EFileStreamText | EFileRead | EFileWrite);
				if (ferr!=KErrNone) {
					TBuf<30> errmsg;
					errmsg.Format(_L("reopen: %d, RESTART NOW"), ferr);
					error(errmsg);
					return;
				}
				*/
				iLog->switch_file();

				transferer->transfer_files(aCommand);
			}
			
		}
		break;
	}
}


void CContextLocaAppUi::DialogTest()
{
	return;
}

void CContextLocaAppUi::SettingChanged(TInt Setting)
{
	if (Setting==SETTING_LOGGING_ENABLE) {
		TApaTaskList tl(Ws());
		TApaTask booktask=tl.FindApp(KUidcontextbook);
		if (booktask.Exists()) {
			booktask.SendSystemEvent(EApaSystemEventShutdown);
		}
		TBool logging;
		Settings().GetSettingL(SETTING_LOGGING_ENABLE, logging);
		if (iLoggingRunning) {	
			if (!logging) {
				iLoggingRunning->SetCurrentState( EMbmContextlocaL_not, EMbmContextlocaL_not );
			} else {
				iLoggingRunning->SetCurrentState( EMbmContextlocaL, EMbmContextlocaL );
			}
		}

	}
}
// End of File  
