/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/

// INCLUDE FILES
#include "Context_logDocument.h"
#include "Context_logAppUi.h"
#include "app_context.h"
#include "settings.h"
#include "cl_settings.h"
#include "symbian_auto_ptr.h"
#include "bb_settings.h"

#include "cc_schedulers.h"
#include <bautils.h>
#include "callstack.h"

#define DO_DEBUG
// ================= MEMBER FUNCTIONS =======================

// constructor
CContext_logDocument::CContext_logDocument(CEikApplication& aApp)
: CAknDocument(aApp)    
{
}

// destructor
CContext_logDocument::~CContext_logDocument()
{
}

void MoveSingleFileL(RFs& aFs, const TDesC& aName)
{
	TFileName from, to;
	from=_L("e:\\system\\data\\context\\"); from.Append(aName);
	to=_L("c:\\system\\data\\context\\"); to.Append(aName);
	TInt err=BaflUtils::RenameFile(aFs, from, to);
	if (err!=KErrNone && err!=KErrNotFound) User::Leave(err);
}

void MoveFromMMC(RFs& aFs)
{
	MoveSingleFileL(aFs, _L("CELLS.db"));
	MoveSingleFileL(aFs, _L("TUPLE.db"));
	MoveSingleFileL(aFs, _L("TAGS.db"));
	MoveSingleFileL(aFs, _L("CL_MEDIA.db"));
	MoveSingleFileL(aFs, _L("TRANSFER.db"));
	MoveSingleFileL(aFs, _L("cellid_names_v3.txt"));
	MoveSingleFileL(aFs, _L("BTDEV.db"));
	MoveSingleFileL(aFs, _L("BASESTACK.db"));
	MoveSingleFileL(aFs, _L("BASES.db"));
	MoveSingleFileL(aFs, _L("LOOKUPS.db"));
}

// EPOC default constructor can leave.
void CContext_logDocument::ConstructL()
{
	MContextDocument::ConstructL(Application(),
		iDefaultSettings, KCLSettingsTuple,
		_L("context_log.txt"), _L("context_log"));

#ifdef __WINS__
	iContext->SetAppDir(_L("c:\\system\\apps\\context_log\\"));
#endif
}

// Two-phased constructor.
CContext_logDocument* CContext_logDocument::NewL(
						 CEikApplication& aApp)     // CContext_logApp reference
{
	auto_ptr<CContext_logDocument> self(new (ELeave) CContext_logDocument( aApp ));
	self->ConstructL();
	
	return self.release();
}

// ----------------------------------------------------
// CContext_logDocument::CreateAppUiL()
// constructs CContext_logAppUi
// ----------------------------------------------------
//
CEikAppUi* CContext_logDocument::CreateAppUiL()
{
	return new (ELeave) CContext_logAppUi(*iContext);
}

// End of File  
