#include "cellnaming.h"
#include "symbian_tree.h"
#include "current_loc.h"
#include "locationing.h"
#include "alerter.h"
#include "csd_cell.h"
#include <AknQueryDialog.h> 
#include "app_context_fileutil.h"
#include <f32file.h>
#include <bautils.h>
#include "mcc.h"
#include "cl_settings.h"
#include "sensorrunner.h"
#include "cellmap.h"
#include "csd_presence.h"
#include "contextcommon.h"
#include "raii_f32file.h"
#include "concretedata.h"

#ifndef FLICKR
#include <context_log.rsg>
#else
#include <contextflickr.rsg>
#endif

#include "emptytext.h"
#include "contextappui.h"
#include "break.h"
#include "symbian_auto_ptr.h"
#include "csd_base.h"
#include <contextui.rsg>

_LIT(KCellAnnotation, "cell_ann");
_LIT(cellid_file_v1, "cellid_names.txt");
_LIT(cellid_file_v2, "cellid_names_v2.txt");
_LIT(cellid_file, "cellid_names_v3.txt");
_LIT(install_cellid_file, "cellid_names_inst_v3.txt");
_LIT(transfer_cellid_file, "cellid_names_trans_v3.txt");


class name_cell_callback : public CCheckedActive {
public:
	name_cell_callback(class CCellNamingImpl* i_ui);
	~name_cell_callback();
	void ConstructL();
	void call();
private:
	void CheckedRunL();
	void DoCancel();
	class CCellNamingImpl* ui;
	RTimer timer;
};

void delete_bufc(void* data)
{
	HBufC* p;
	p=(HBufC*)data;
	delete p;
}

class CCellNamingImpl : public CCellNaming, public MContextBase, public MNaming, public Mlogger {
private:

	CGenericIntMap* cellid_names;
	CCurrentLoc*	iCurrentLoc;
	MNaming&	iNaming;
	CAlerter*	iAlerter;
	name_cell_callback* caller;
	CArrayFixFlat<TTime>* name_times;
	CArrayFixFlat<TBBCellId>* name_names;

	TInt iCurrentBaseId, iCurrentCellId;
	TBBLongString iCity;
	TBBShortString iCountry;
	TBBLongString iCell;
	TBool iSeenLoc;
	CSensorRunner*	iSensorRunner;
	MContextAppUi&	iAppUi;
	CCnvCharacterSetConverter* iCC;
	TInt iResource;

	CCellNamingImpl(MApp_context& aContext, MNaming& aNaming, MContextAppUi& aAppUi) : MContextBase(aContext),
		iNaming(aNaming), iCity(KCity), iCountry(KCountry), iCell(KCellName), iAppUi(aAppUi) { }

	void ConstructL(class sms* smsh, i_status_notif* i_cb, MNetworkConnection * aPublisher,
		CSensorRunner* aSensorRunner) {
#if !defined(REALLY_ONLY_LOGGING) || defined(DO_LOCATIONING)
		if (smsh) loc=Clocationing::NewL(AppContext(), i_cb, this, smsh, aPublisher);
#endif
		iSensorRunner=aSensorRunner;
		Mlogger::ConstructL(AppContextAccess());
		SubscribeL(KLocationTuple);
		SubscribeL(KCellNameTuple);
		SubscribeL(KCityNameTuple);
		SubscribeL(KIncomingPresence);

#ifndef __WINS__
		TFileName resfile=_L("c:\\System\\data\\contextui.rsc");
		if (! BaflUtils::FileExists(CEikonEnv::Static()->FsSession(), resfile) ) {
			resfile.Replace(0, 1, _L("e"));
		}
#else
		TFileName resfile=_L("z:\\System\\data\\contextui.rsc");
		BaflUtils::NearestLanguageFile(CEikonEnv::Static()->FsSession(), resfile); //for localization
#endif
		iResource=CEikonEnv::Static()->AddResourceFileL(resfile);

		iCC=CCnvCharacterSetConverter::NewL();
		convert_v2_to_v3();

		TUint cellid_file_version=2;
		{
			MoveIntoDataDirL(AppContext(), cellid_file);
			// copy the installed cellid_names over the real one, if
			// one has been installed

			TFileName install_cellid_filen, cellid_filen, cellid_filen_v1;
			install_cellid_filen.Format(_L("%S%S"), &AppDir(), &install_cellid_file);
			cellid_filen.Format(_L("%S%S"), &DataDir(), &cellid_file);
			cellid_filen_v1.Format(_L("%S%S"), &DataDir(), &cellid_file_v1);
			if (BaflUtils::FileExists(Fs(), install_cellid_filen)) {
				User::LeaveIfError(BaflUtils::CopyFile(Fs(), install_cellid_filen, cellid_filen));
				User::LeaveIfError(BaflUtils::DeleteFile(Fs(), install_cellid_filen));
			}
			TEntry e;
			if ( ( Fs().Entry(cellid_filen, e)!=KErrNone || e.iSize==0 ) 
					&& cellid_name_file_v1.Open(Fs(), cellid_filen_v1, 
					EFileShareAny | EFileStreamText | EFileRead | EFileWrite)==KErrNone) {
				cellid_file_version=1;
			}

			if (cellid_name_file.Open(Fs(), cellid_filen, 
				EFileShareAny | EFileStreamText | EFileRead | EFileWrite)!=KErrNone) {
				User::LeaveIfError(cellid_name_file.Create(Fs(), cellid_filen, 
					EFileShareAny | EFileStreamText | EFileRead | EFileWrite));
			}
		}


		cellid_names=CGenericIntMap::NewL();
		cellid_names->SetDeletor(delete_bufc);

		iCurrentLoc=CCurrentLoc::NewL(AppContext(), cellid_names);

		iAlerter=CAlerter::NewL(AppContext());
		cell_ann=Clog_base_impl::NewL(AppContext(), KCellAnnotation, KCellAnnotationTuple, 0);
		city_name=Clog_base_impl::NewL(AppContext(), KCity, KCityNameTuple, 10*60+30);
		cell_name=Clog_base_impl::NewL(AppContext(), KCellName, KCellNameTuple, 10*60+30);
		country_name=Clog_base_impl::NewL(AppContext(), KCountry, KCountryNameTuple, 10*60+30);

		caller=new (ELeave) name_cell_callback(this);
		caller->ConstructL();

		name_times=new (ELeave) CArrayFixFlat<TTime>(2);
		name_names=new (ELeave)CArrayFixFlat<TBBCellId>(2);


		if (cellid_file_version==1) {
			iCC->PrepareToConvertToOrFromL(KCharacterSetIdentifierIso88591, Fs());
			read_cellid_names(cellid_file_version);
			cellid_name_file_v1.Close();
		} else {
			iCC->PrepareToConvertToOrFromL(KCharacterSetIdentifierUtf8, Fs());
			read_cellid_names(cellid_file_version);
		}

		TInt pos=0;
		cellid_name_file.Seek(ESeekEnd, pos);

	}

	Mlog_base_impl * cell_ann, *city_name, *country_name, * cell_name;
	~CCellNamingImpl() {

		if (iResource) CEikonEnv::Static()->DeleteResourceFile(iResource);

		delete loc;
		delete cellid_names;
		delete iCurrentLoc;
		delete iAlerter;
		delete cell_ann;
		delete city_name;
		delete country_name;
		delete cell_name;
		cellid_name_file.Close();
		delete name_times; delete name_names;
		delete caller;
		delete iCC;
	}

	void add_name_to_map(const TBBCellId& Cell, const TDesC& name) {
		HBufC* n=HBufC::NewL(name.Length());
		CleanupStack::PushL(n);
		*n=name;
		TInt id=Cell.iMappedId();
		cellid_names->AddDataL(id, (void*)n, true); 
		CleanupStack::Pop(); // n
	}
	void add_cellid_name(const TBBCellId& Cell, const TDesC& name) {
		if (Cell.iCellId()==0) {
			if (name.Length()>iCity.Value().MaxLength()) {
				iCity()=name.Left(iCity.Value().MaxLength());
			} else {
				iCity()=name;
			}
			city_name->post_new_value(&iCity);
		} else {
			if (name.Length()>iCell.Value().MaxLength()) {
				iCell()=name.Left(iCell.Value().MaxLength());
			} else {
				iCell()=name;
			}
			cell_name->post_new_value(&iCell);
		}
		TBuf<50> b;
		Cell.IntoStringL(b);

		write_to_output(b);
		_LIT(tab, "\t");
		write_to_output(tab);
		write_to_output(name);
		_LIT(nl, "\n");
		write_to_output(nl);

		_LIT(KCellAnn, "cell_ann");
		TBBShortString m(name, KCellAnn);
		cell_ann->post_new_value(&m);

		add_name_to_map(Cell, name);
		TInt id=Cell.iMappedId();
		if (id==iCurrentLoc->CurrentBaseId()) {
			TTime now=GetTime();
			iCurrentLoc->now_at_location(&Cell, id, true, false, now);
		}
		iNaming.add_cellid_name(Cell, name);
	}
	bool is_named(TInt id)
	{
		CALLSTACKITEM_N(_CL("CContext_logContainer"), _CL("is_named"));

		if (cellid_names->GetData(id)) return true;
		return false;
	}


	CCircularLog*	BaseLog()
	{
		return iCurrentLoc->BaseLog();
	}

	CGenericIntMap* get_cellid_names()
	{
		return cellid_names;
	}
	CCurrentLoc* CurrentLoc()
	{
		return iCurrentLoc;
	}


	enum ui_state { IDLE, NAME_CELLS };
	ui_state current_state;

	Clocationing* loc;

	void NewSensorEventL(const TTupleName& aName, 
						const TDesC& aSubName, const CBBSensorEvent& aEvent)
	{
		if (aName==KLocationTuple) {
			const TBBLocation* loc=bb_cast<TBBLocation>(aEvent.iData());
			if (!loc) return;
			now_at_location(&(loc->iCellId), loc->iLocationId(), loc->iIsBase(), loc->iLocationChanged(),
				loc->iEnteredLocation());
		} else if (aName==KCellNameTuple || aName==KCityNameTuple) {
			const TBBFixedLengthStringBase* name=0;
			name=bb_cast<TBBShortString>(aEvent.iData());
			if (!name) name=bb_cast<TBBLongString>(aEvent.iData());
			if (!name) return;

			TBBCellId ask_for(KCellName);
			TInt id=iCurrentBaseId;
			if (id!=-1) {
				ask_for.iMappedId()=id;
				TRAPD(err,
					iSensorRunner->CellMap()->GetCellL(id, ask_for));
				if (err==KErrNotFound) id=-1;
				else if (err!=KErrNone) return;
			}
			if (id==-1) {
				id=iCurrentCellId;
				ask_for.iMappedId()=id;
				TRAPD(err,
					iSensorRunner->CellMap()->GetCellL(id, ask_for));
				if (err!=KErrNone) return;
			}
			if (aName==KCellNameTuple) {
				HBufC* existing_name=(HBufC*)get_cellid_names()->GetData( id );
				if (!existing_name || existing_name->Compare(name->Value())) {
					add_cellid_name(ask_for, name->Value());
				}
			} else {
				ask_for.iCellId()=0;
				TRAPD(err,
					id=iSensorRunner->CellMap()->GetId(ask_for));
				if (id==-1 || err!=KErrNone) return;
				ask_for.iMappedId()=id;
				HBufC* existing_name=(HBufC*)get_cellid_names()->GetData( id );
				if (!existing_name || existing_name->Compare(name->Value())) {
					add_cellid_name(ask_for, name->Value());
				}
			}
		}
	}
	virtual void NewValueL(TUint , const TTupleName& aName, 
		const TDesC& aSubName, 
		const TComponentName& /*aComponent*/,
		const MBBData* aData) {
		const CBBSensorEvent* e=bb_cast<CBBSensorEvent>(aData);
		if (e) {
			NewSensorEventL(aName, aSubName, *e);
		} else if (aName==KIncomingPresence) {
			const CBBPresence* pres=bb_cast<CBBPresence>(aData);
			if (!pres) return;

			TBBCellId cell=pres->iCellId;
			if (cell.iMCC()==0 || cell.iMNC()==0 || cell.iLocationAreaCode()==0) return;
			if (pres->iCellName().Length()>0) {
				TInt id=-1;
				TRAPD(err,
					id=iSensorRunner->CellMap()->GetId(cell));
				if (err==KErrNone && id>-1 && !is_named(id)) {
					cell.iMappedId()=id;
					add_cellid_name(cell, pres->iCellName());
				}
			}
			if (pres->iCity().Length()>0) {
				TInt id=-1;
				cell.iCellId()=0;
				TRAPD(err,
					id=iSensorRunner->CellMap()->GetId(cell));
				if (err==KErrNone && id>-1 && !is_named(id)) {
					cell.iMappedId()=id;
					add_cellid_name(cell, pres->iCity());
				}
			}
		}
	}

	void ask_operator(const class TBBCellId & cell) {
		loc->GetNameL(&cell, ETrue);
	}

	TBool locationing_available() {
		return (loc && loc->LocationingAvailable());
	
	}
	void now_at_location(const TBBCellId* Cell, TInt id, 
						bool is_base, bool loc_changed, TTime time)
	{
		if (loc) loc->now_at_location(Cell, id, is_base, loc_changed, time);
		iCurrentLoc->now_at_location(Cell, id, is_base, loc_changed, time);
		if (id<0 || !Cell) return;

		GetCountryName( Cell->iMCC(), iCountry() );
		country_name->post_new_value(&iCountry);

		TBool ask_for_name=EFalse, get_name=EFalse;
		if ( loc_changed || ! iSeenLoc) {
			// don't prompt/get name from operator multiple times
			// in the same location
			Settings().GetSettingL(SETTING_PROMPT_FOR_NAMES, ask_for_name);
			get_name=ETrue;
		}
		iSeenLoc=ETrue;
		TTime now;
		now.HomeTime();
		TBool do_call=EFalse;

		TInt id_for_cell=Cell->iMappedId();
		iCurrentCellId=id_for_cell;
		iCurrentBaseId=id;

		_LIT(KCell, "cell");

		HBufC* cellname=0;
		cellname=(HBufC*)get_cellid_names()->GetData( id );
		if (!cellname) cellname=(HBufC*)get_cellid_names()->GetData( id_for_cell );
		if (cellname) {
			iCell()=cellname->Des().Left(iCell().MaxLength());
		} else {
			iCell().Zero();
		}
		cell_name->post_new_value(&iCell);

		if (is_base && ! is_named(id) && ! is_named(id_for_cell) ) {
			TBBCellId ask_for(KCell);
			ask_for.iMappedId()=id;
			iSensorRunner->CellMap()->GetCellL(id, ask_for);
			if (get_name && loc && loc->LocationingAvailable() && loc->LocationingAllowed()) {
				loc->GetNameL(Cell);
			} else {
				if (ask_for_name) {
					// don't insert duplicates
					TBool found=EFalse;
					for (int i=0; i < name_names->Count(); i++) {
						if ( (*name_names)[i]==*Cell ) {
							found=ETrue;
							break;
						}
					}
					if (! found ) {
						name_times->AppendL(now);
						name_names->AppendL(*Cell);
						do_call=ETrue;
					}
				}
			}
		}
		TBBCellId city=*Cell; city.iCellId()=0;
		city.iMappedId()=iSensorRunner->CellMap()->GetId(city);
		HBufC* cityname=(HBufC*)get_cellid_names()->GetData( city.iMappedId() );
		if (cityname) {
			iCity()=cityname->Des().Left(iCity().MaxLength());
		} else {
			iCity().Zero();
		}
		city_name->post_new_value(&iCity);

		if ( ask_for_name && ! cityname ) {
			TBool found=EFalse;
			for (int i=0; i < name_names->Count(); i++) {
				if ( (*name_names)[i]==city ) {
					found=ETrue;
					break;
				}
			}
			if (! found ) {
				name_times->AppendL(now);
				name_names->AppendL(city);
				do_call=ETrue;
			}
		}

		if (do_call) caller->call();
	}

	void CCellNamingImpl::name_cell(const CAknQueryDialog::TTone tone, const TBBCellId & cell, TTime time)
	{
		if ( ((CCoeAppUi*)CEikonEnv::Static()->AppUi())->IsDisplayingMenuOrDialog()) return;

		const TInt NAME_LENGTH=100;
		auto_ptr<HBufC> textData(HBufC::NewL(NAME_LENGTH));

		HBufC* existing_name=(HBufC*)cellid_names->GetData(cell.iMappedId());
		if (existing_name) textData->Des()=*existing_name;
		
		TPtr16 p=textData->Des();

		auto_ptr<HBufC> f(0);
		if (time!=TTime(0)) {
			if (cell.iCellId()!=0) {
				f.reset(CEikonEnv::Static()->AllocReadResourceL(R_CL_NAME_CELL_CAPTION));
			} else {
				// city
				f.reset(CEikonEnv::Static()->AllocReadResourceL(R_CL_NAME_CITY_CAPTION));
			}
		} else {
			if (cell.iCellId()!=0) {
				f.reset(CEikonEnv::Static()->AllocReadResourceL(R_CL_NAME_OLD_CELL_CAPTION));
			} else {
				// city
				f.reset(CEikonEnv::Static()->AllocReadResourceL(R_CL_NAME_OLD_CITY_CAPTION));
			}
		}
		auto_ptr<HBufC> pr(HBufC::NewL(f->Des().Length()+4));
		if (time!=TTime(0)) {
			TDateTime dt(time.DateTime());
			pr->Des().Format(f->Des(), dt.Hour(), dt.Minute());
		} else {
			pr->Des()=*f;
		}

		CAknTextQueryDialog* dlg = new(ELeave) CEmptyAllowingTextQuery(p, tone);
		CleanupStack::PushL(dlg);
		dlg->SetMaxLength(NAME_LENGTH);
		dlg->SetPromptL(*pr);
		CleanupStack::Pop();
		if (dlg->ExecuteLD(R_CONTEXT_LOG_NAME_INPUT) && textData->Length())
		{
			add_cellid_name(cell, *textData);
		}
	}

	void name_cells()
	{	
		CALLSTACKITEM_N(_CL("CCellNamingImpl"), _CL("name_cells"));

		RWsSession& wsSession=CEikonEnv::Static()->WsSession();
		TInt id = CEikonEnv::Static()->RootWin().Identifier();
		TApaTask task(wsSession);
		task.SetWgId(id);
		TBool hide_me = EFalse;
		if ( wsSession.GetFocusWindowGroup() != id )
		{
			task.BringToForeground();
			hide_me = ETrue;
		}
		//unhide();

		current_state=NAME_CELLS;

		// cannot play tone if we want to play a ringtone
		CAknQueryDialog::TTone tone=CAknQueryDialog::ENoTone; 
		iAlerter->AlertUser();

		{
			for (int i=0; i<name_names->Count(); i++) {
				TBBCellId& cell=(*name_names)[i];
				name_cell(tone, cell, (*name_times)[i]);
				name_times->Delete(i);
				name_names->Delete(i);
			}
			name_times->Compress();
			name_names->Compress();
		}

		if (iAlerter) iAlerter->StopAlert();

		if (hide_me) 
		{
			iAppUi.hide();
		}
		current_state=IDLE;

	}

	RFile cellid_name_file, cellid_name_file_v1;
	TBuf8<128> buf;
	
	void convert_v2_to_v3()
	{
		auto_ptr<HBufC> cellid_filen(HBufC::NewL(256));
		auto_ptr<HBufC> cellid_filen_v2(HBufC::NewL(256));
		cellid_filen->Des().Format(_L("%S%S"), &DataDir(), &cellid_file);
		cellid_filen_v2->Des().Format(_L("%S%S"), &DataDir(), &cellid_file_v2);
		RFile v2; TEntry e;
		if ( ( Fs().Entry(*cellid_filen_v2, e)==KErrNone || e.iSize>0 ) 
				&& v2.Open(Fs(), *cellid_filen_v2, 
				EFileShareAny | EFileStreamText | EFileRead | EFileWrite)==KErrNone ) {
			CleanupClosePushL(v2);
			{
				auto_ptr<CCnvCharacterSetConverter> utf8CC(CCnvCharacterSetConverter::NewL());
				utf8CC->PrepareToConvertToOrFromL(KCharacterSetIdentifierUtf8, Fs());
				auto_ptr<CCnvCharacterSetConverter> latin1CC(CCnvCharacterSetConverter::NewL());
				latin1CC->PrepareToConvertToOrFromL(KCharacterSetIdentifierIso88591, Fs());
				RAFile v3; v3.ReplaceLA(Fs(), *cellid_filen, EFileWrite | EFileShareAny | EFileStreamText);
				auto_ptr<HBufC8> latin1(HBufC8::NewL(512));
				auto_ptr<HBufC8> utf8(HBufC8::NewL(1024+10));
				auto_ptr<HBufC> unic(HBufC::NewL(512));
				TInt state=CCnvCharacterSetConverter::KStateDefault;
				TPtr8 latin1p=latin1->Des();
				while(v2.Read(latin1p)==KErrNone && latin1->Des().Length()>0) {
					TPtr unicp=unic->Des();
					latin1CC->ConvertToUnicode(unicp, latin1p, state);
					TPtr8 utf8p=utf8->Des();
					utf8CC->ConvertFromUnicode(utf8p, *unic);
					User::LeaveIfError(v3.Write(*utf8));
					unic->Des().Zero();
					latin1->Des().Zero();
					utf8->Des().Zero();
					latin1p.Set(latin1->Des());
				}
			}
			CleanupStack::PopAndDestroy();
		}
		TInt err=Fs().Delete(*cellid_filen_v2);
		if (err!=KErrNone && err!=KErrNotFound) User::Leave(err);
	}
	void read_cellid_names(TUint version)
	{
		CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("read_cellid_names"));

		TBuf<128> unibuf;
		TBuf<200> cell_name, cellid;
		TInt pos=0; TChar cur; TInt len;
		bool read_id=false;

		TInt state=CCnvCharacterSetConverter::KStateDefault;
		if (version==1) {
			cellid_name_file_v1.Read(buf);
		} else {
			cellid_name_file.Read(buf);
		}
		while ( (len=buf.Length()) ) {
			iCC->ConvertToUnicode(unibuf, buf, state);
			len=unibuf.Length();
			while (pos<len) {
				while (!read_id && pos<len && (cur=unibuf[pos])!=L'\t') {
					cellid.Append(cur);
					pos++;
				}
				if (pos<len && !read_id) {
					read_id=true;
					pos++; // skip tab
				}
				while (pos<len && (cur=unibuf[pos])!=L'\n') {
					cell_name.Append(cur);
					pos++;
				}
				if (pos<len) {
					pos++; //skip nl for next
					
					if (cellid.Length() && cell_name.Length()) {
						TBBCellId cell(KCell);

						if (version==1) {
							TUint id, lac;
							CCellMap::Parse(cellid, id, lac, cell.iShortName());
							cell.iCellId()=id; cell.iLocationAreaCode()=lac;
							iSensorRunner->OpMap()->NameToMccMnc(cell.iShortName(), cell.iMCC(), cell.iMNC());
							cell.iMappedId()=iSensorRunner->CellMap()->GetId(cell);
							add_name_to_map(cell, cell_name);
						} else {
							CC_TRAPD(err, cell.FromStringL(cellid));
							if (err==KErrNone) {
								cell.iMappedId()=iSensorRunner->CellMap()->GetId(cell);
								add_name_to_map(cell, cell_name);
							}
						}
					}
					cellid.Zero();
					cell_name.Zero();
					read_id=false;
				}
			}
			buf.Zero();
			pos=0;
			if (version==1) {
				cellid_name_file_v1.Read(buf);
			} else {
				cellid_name_file.Read(buf);
			}
		}
	}

	void write_to_output(const TDesC& str)
	{
		CALLSTACKITEM_N(_CL("CContext_logAppUi"), _CL("write_to_output"));

		TInt pos=0, len;
		len=buf.MaxLength();
		while (pos < str.Length()) {
			TInt real_len;
			if (pos+len > str.Length()) {
				real_len=str.Length()-pos;
			} else {
				real_len=len;
			}
			iCC->ConvertFromUnicode(buf, str.Mid(pos, real_len));
			pos+=len;
			cellid_name_file.Write(buf);
		}

	}


	friend name_cell_callback;
	friend CCellNaming;
	friend auto_ptr<CCellNamingImpl>;
};

CCellNaming* CCellNaming::NewL(class MApp_context& aContext, class sms* smsh,
	class i_status_notif* i_cb, class MNetworkConnection * aPublisher,
	class MNaming& aNaming, class CSensorRunner* aSensorRunner,
	class MContextAppUi& aAppUi)
{
	auto_ptr<CCellNamingImpl> ret(new (ELeave) CCellNamingImpl(aContext, aNaming, aAppUi));
	ret->ConstructL(smsh, i_cb, aPublisher, aSensorRunner);
	return ret.release();
}

name_cell_callback::name_cell_callback(CCellNamingImpl* i_ui) : 
	CCheckedActive(EPriorityIdle, _L("name_cell_callback")), ui(i_ui) { }

void name_cell_callback::ConstructL()
{
	CALLSTACKITEM_N(_CL("name_cell_callback"), _CL("ConstructL"));

	timer.CreateLocal();
	CActiveScheduler::Add(this); // add to scheduler
}

void name_cell_callback::call()
{
	CALLSTACKITEM_N(_CL("name_cell_callback"), _CL("call"));

	if (IsActive()) return;
	TTimeIntervalMicroSeconds32 w(1*1000*1000);
	timer.After(iStatus, w);
	SetActive();
}

void name_cell_callback::CheckedRunL()
{
	CALLSTACKITEM_N(_CL("name_cell_callback"), _CL("CheckedRunL"));

	if (ui) {
		ui->name_cells();
	}
}

void name_cell_callback::DoCancel()
{
	CALLSTACKITEM_N(_CL("name_cell_callback"), _CL("DoCancel"));

	timer.Cancel();
}

name_cell_callback::~name_cell_callback()
{
	CALLSTACKITEM_N(_CL("name_cell_callback"), _CL("~name_cell_callback"));

	Cancel();
	timer.Close();
}
