#include "presencemaintainer.h"
#include "csd_cell.h"
#include "csd_profile.h"
#include "csd_bluetooth.h"
#include "csd_base.h"
#include "csd_idle.h"
#include "csd_gps.h"
#include "csd_presence.h"
#include "btlist.h"
#include "cm_autotaglist.h"
#include "symbian_auto_ptr.h"
#include "cl_settings.h"
#include "break.h"
#include "csd_unread.h"

MPresenceMaintainer::MPresenceMaintainer(MApp_context& aContext,CBTDeviceList* aBuddyList,
				         CBTDeviceList *aLaptopBTs, CBTDeviceList *aDesktopBTs, 
				         CBTDeviceList *aPDABTs, class MAskForNames* aCellNames) : 
	MContextBase(aContext),
	iBuddyList(aBuddyList), iLaptopBTs(aLaptopBTs), iDesktopBTs(aDesktopBTs), iPDABTs(aPDABTs),
	iCellNames(aCellNames) { }

void MPresenceMaintainer::ConstructL()
{
	Mlogger::ConstructL(AppContextAccess());
	SubscribeL(KAnySensorEvent);

	iData=CBBPresence::NewL();
	iNonsignificantTimer=CTimeOut::NewL(*this);

	MBBData *gps_event_data=0;
	TInt err;
	CC_TRAP(err, iBBSubSessionNotif->GetL(KLastKnownGpsTuple, KNullDesC, gps_event_data));
	if (gps_event_data) {
		const CBBSensorEvent* aEvent=bb_cast<CBBSensorEvent>(gps_event_data);
		if (aEvent) {
			const TGpsLine* gps=bb_cast<TGpsLine>(aEvent->iData());
			if (gps) {
				iData->iGps=*gps;
				iData->iGpsStamp()=aEvent->iStamp();
			}
		}
	}
	delete gps_event_data;

}

MPresenceMaintainer::~MPresenceMaintainer()
{
	delete iData;
	delete iNonsignificantTimer;
}

// Mlogger

void MPresenceMaintainer::DeletedL(const TTupleName& aName, const TDesC& aSubName)
{
	CALLSTACKITEM_N(_CL("MPresenceMaintainer"), _CL("NewSensorEventL"));

	const TTupleName& tuplename=aName;
	TBool frozenChanged=EFalse;

	if (tuplename==KCellIdTuple) {
		iData->iCellId=TBBCellId(KCell);
		iData->iCellName().Zero();
		return;
	} else if (tuplename==KCellNameTuple) {
		iData->iCellName().Zero();
		return;
	} else if (tuplename==KUserGivenContextTuple) {
		iData->iUserGiven.iDescription().Zero();
		iData->iUserGiven.iSince()=GetTime();
		//Settings().WriteSettingL(SETTING_OWN_DESCRIPTION, iData->iUserGiven.iDescription());
		//Settings().WriteSettingL(SETTING_OWN_DESCRIPTION_TIME, iData->iUserGiven.iSince());

	} else if (tuplename==KProfileTuple) {
		iData->iProfile=TBBProfile();
	} else if (tuplename==KIdleTuple ) {
		iData->iUserActive=TBBUserActive();
	} else if (tuplename==KLastKnownGpsTuple ) {
		iData->iGps=TGpsLine();
		return;
	} else if (tuplename==KBaseTuple) {
		bb_auto_ptr<TBBBaseInfo> b(new (ELeave) TBBBaseInfo());
		iData->iBaseInfo=*b;
	} else if (tuplename==KCalendarTuple) {
		iData->iCalendar=TBBCalendar();
		if (iFrozen) iFrozen->iCalendar=TBBCalendar();
		frozenChanged=ETrue;

	} else if (tuplename==KCountryNameTuple) {
		iData->iCountry().Zero();

	} else if (tuplename==KCityNameTuple) {
		iData->iCity().Zero();

	} else if (tuplename==KBluetoothTuple) {
		iData->iDevices->Reset();
		iData->iNeighbourhoodInfo=TBBNeighbourhoodInfo();
	} else if (tuplename==KAlarmTuple) {
		iData->iAlarm()=TTime(0);
	} else if (tuplename==KUnreadTuple) {
		iData->iUnread=TBBUnread();
	} else {
		return;
	}

	iLastUpdateTime=GetTime();
	iNonsignificantTimer->Reset();
	GotSignificantChange(frozenChanged);
}

void MPresenceMaintainer::NewSensorEventL(const TTupleName& aName, 
					 const TDesC& , const CBBSensorEvent& aEvent)
{
	CALLSTACKITEM_N(_CL("MPresenceMaintainer"), _CL("NewSensorEventL"));

	if (aEvent.iPriority()!=CBBSensorEvent::VALUE || !aEvent.iData()) return;

	const TTupleName& tuplename=aName;
	TTime time=aEvent.iStamp();
	TBool frozenChanged=EFalse;

	if (tuplename==KCellIdTuple) {
		const TBBCellId* cell=bb_cast<TBBCellId>(aEvent.iData());
		if (!cell) return;
		iData->iCellId=*cell;
		return;
	} else if (tuplename==KCellNameTuple) {
		TBuf<50> prev_name=iData->iCellName().Left(50);
		iData->iCellName().Zero();
		const TBBLongString* name=bb_cast<TBBLongString>(aEvent.iData());
		if (name) iData->iCellName()=name->Value();
		if (!name || name->Value().Left(50).Compare(prev_name)) {
			TTime now; now=GetTime();
			if ( iLastUpdateTime + TTimeIntervalMinutes(10) > now ) {
				iNonsignificantTimer->WaitMax( 10*60 );
			} else {
				iNonsignificantTimer->Reset();
			}
		} else {
			return;
		}
	} else if (tuplename==KUserGivenContextTuple) {
		const TBBUserGiven * user_given=bb_cast<TBBUserGiven>(aEvent.iData());
		if (!user_given) return;
		iData->iUserGiven=*user_given;
		//Settings().WriteSettingL(SETTING_OWN_DESCRIPTION, user_given->iDescription());
		//Settings().WriteSettingL(SETTING_OWN_DESCRIPTION_TIME, user_given->iSince());

	} else if (tuplename==KProfileTuple) {
		const TBBProfile* profile=bb_cast<TBBProfile>(aEvent.iData());
		if (profile) {
			if (iData->iProfile.Equals(profile)) return;

			iData->iProfile=*profile;
		} else {
			iData->iProfile=TBBProfile();
		}
	} else if (tuplename==KIdleTuple ) {
		const TBBUserActive* act=bb_cast<TBBUserActive>(aEvent.iData());
		if (!act) return;
		if (iData->iUserActive.Equals(act)) return;
		iData->iUserActive=*act;
	} else if (tuplename==KLastKnownGpsTuple ) {
		const TGpsLine* gps=bb_cast<TGpsLine>(aEvent.iData());
		if (!gps) return;
		iData->iGps=*gps;
		iData->iGpsStamp()=aEvent.iStamp();
		return;
	} else if (tuplename==KBaseTuple) {
		const TBBBaseInfo* basep=bb_cast<TBBBaseInfo>(aEvent.iData());
		if (basep) {
			bb_auto_ptr<TBBBaseInfo> base( bb_cast<TBBBaseInfo>(basep->CloneL(basep->Name())) );

			if ( base->iPreviousStay.iLeft() < iUnfreezeTime ) {
				base->iPreviousStay.iBaseName().Zero();
				base->iPreviousStay.iLeft()=iUnfreezeTime;
			}
			if ( base->iPreviousVisit.iLeft() < iUnfreezeTime ) {
				base->iPreviousVisit.iBaseName().Zero();
				base->iPreviousVisit.iLeft()=iUnfreezeTime;
			}
			if ( base->iCurrent.iEntered() < iUnfreezeTime ) {
				base->iCurrent.iEntered()=iUnfreezeTime;
			}
			if (iData->iBaseInfo.Equals(base.get())) return;
			iData->iBaseInfo=*base;
		} else {
			bb_auto_ptr<TBBBaseInfo> base(new (ELeave) TBBBaseInfo());
			iData->iBaseInfo=*base;
		}
	} else if (tuplename==KCalendarTuple) {
		const TBBCalendar* cal=bb_cast<TBBCalendar>(aEvent.iData());
		if (!iFrozen) {
			if (cal) {
				if (iData->iCalendar.Equals(cal)) return;
				iData->iCalendar=*cal;
			} else {
				iData->iCalendar=TBBCalendar();
			}
		} else {
			if (cal) {
				iData->iCalendar=*cal;
				if (iFrozen->iCalendar.Equals(cal)) return;
				iFrozen->iCalendar=*cal;
			} else {
				iFrozen->iCalendar=TBBCalendar();
			}
			frozenChanged=ETrue;
		}

	} else if (tuplename==KCountryNameTuple) {
		const TBBShortString* country=bb_cast<TBBShortString>(aEvent.iData());
		if (country) {
			if (iData->iCountry == *country) return;
			iData->iCountry()=(*country)();
		} else {
			iData->iCountry().Zero();
		}
	} else if (tuplename==KCityNameTuple) {
		const TBBFixedLengthStringBase* City=bb_cast<TBBShortString>(aEvent.iData());
		if (!City) City=bb_cast<TBBLongString>(aEvent.iData());
		if (City) {
			if (iData->iCity == *City) return;
			iData->iCity()=City->Value();
		} else {
			iData->iCity().Zero();
		}
	} else if (tuplename==KAlarmTuple) {
		const TBBTime* alarm=bb_cast<TBBTime>(aEvent.iData());
		if (alarm) {
			if (iData->iAlarm()==(*alarm)()) return;
			iData->iAlarm()=(*alarm)();
		} else {
			iData->iAlarm()=TTime(0);
		}
	} else if (tuplename==KUnreadTuple) {
		const TBBUnread* unread=bb_cast<TBBUnread>(aEvent.iData());
		if (unread) {
			if (iData->iUnread==*unread) return;
			iData->iUnread=*unread;
		} else {
			iData->iUnread=TBBUnread();
		}
	} else if (tuplename==KBluetoothTuple) {
		const CBBBtDeviceList* devs=bb_cast<CBBBtDeviceList>(aEvent.iData());
		if (!devs) return;

		iData->SetDevices(devs);

		TUint prev_buddies=iData->iNeighbourhoodInfo.iBuddies();
		TUint prev_others=iData->iNeighbourhoodInfo.iOtherPhones();
		TUint prev_laptops=iData->iNeighbourhoodInfo.iLaptops();
		TUint prev_desktops=iData->iNeighbourhoodInfo.iDesktops();
		TUint prev_pdas=iData->iNeighbourhoodInfo.iPDAs();
		iData->iNeighbourhoodInfo.iBuddies()=0;
		iData->iNeighbourhoodInfo.iOtherPhones()=0;
		iData->iNeighbourhoodInfo.iLaptops()=0;
		iData->iNeighbourhoodInfo.iDesktops()=0;
		iData->iNeighbourhoodInfo.iPDAs()=0;

		for (const TBBBtDeviceInfo *node=devs->First(); node; node=devs->Next()) {
			if ( iBuddyList && iBuddyList->ContainsDevice(node->iMAC()) ) {
				++iData->iNeighbourhoodInfo.iBuddies();
			} else if ( iLaptopBTs && iLaptopBTs->ContainsDevice(node->iMAC()) ) {
				++iData->iNeighbourhoodInfo.iLaptops();
			} else if ( iDesktopBTs && iDesktopBTs->ContainsDevice(node->iMAC()) ) {
				++iData->iNeighbourhoodInfo.iDesktops();
			} else if ( iPDABTs && iPDABTs->ContainsDevice(node->iMAC()) ) {
				++iData->iNeighbourhoodInfo.iPDAs();
			} else if ( node->iMajorClass() == 2 ) {
				++iData->iNeighbourhoodInfo.iOtherPhones();
			} 
		}
#ifdef __WINS__
		iData->iNeighbourhoodInfo.iLaptops()=1;
		iData->iNeighbourhoodInfo.iDesktops()=1;
		iData->iNeighbourhoodInfo.iPDAs()=1;
#endif
		if (iData->iNeighbourhoodInfo.iBuddies()==prev_buddies &&
			iData->iNeighbourhoodInfo.iOtherPhones()==prev_others &&
			iData->iNeighbourhoodInfo.iLaptops()==prev_laptops &&
			iData->iNeighbourhoodInfo.iDesktops()==prev_desktops &&
			iData->iNeighbourhoodInfo.iPDAs()==prev_pdas) return;

	}

	iLastUpdateTime=GetTime();
	iNonsignificantTimer->Reset();
	GotSignificantChange(frozenChanged);
}

void MPresenceMaintainer::add_cellid_name(const TBBCellId& cellid, const TDesC& name)
{
	if (cellid==iData->iCellId) iData->iCellName()=name;
}

void MPresenceMaintainer::expired(CBase* Source)
{
	iLastUpdateTime=GetTime();
	GotSignificantChange(EFalse);
}

const CBBPresence* MPresenceMaintainer::Data() const
{
	return iData;
}

CPresenceMaintainer::CPresenceMaintainer(MApp_context& aContext, CBTDeviceList* aBuddyList,
				  CBTDeviceList *aLaptopBTs, CBTDeviceList *aDesktopBTs, 
				  CBTDeviceList *aPDABTs, class MAskForNames* aCellNames) :
	MPresenceMaintainer(aContext, aBuddyList, aLaptopBTs, aDesktopBTs,
		aPDABTs, aCellNames) { }

class CPresenceMaintainerImpl : public CPresenceMaintainer {
private:
	virtual void GotSignificantChange(TBool frozenChanged) { }
	CPresenceMaintainerImpl(MApp_context& aContext, CBTDeviceList* aBuddyList,
					  CBTDeviceList *aLaptopBTs, CBTDeviceList *aDesktopBTs, 
					  CBTDeviceList *aPDABTs, class MAskForNames* aCellNames) :
	CPresenceMaintainer(aContext, aBuddyList, aLaptopBTs, aDesktopBTs,
		aPDABTs, aCellNames) { }

	friend CPresenceMaintainer;
	friend auto_ptr<CPresenceMaintainerImpl>;
};

CPresenceMaintainer* CPresenceMaintainer::NewL(MApp_context& aContext, CBTDeviceList* aBuddyList,
				  CBTDeviceList *aLaptopBTs, CBTDeviceList *aDesktopBTs, 
				  CBTDeviceList *aPDABTs, class MAskForNames* aCellNames)
{
	auto_ptr<CPresenceMaintainerImpl> ret(new (ELeave) CPresenceMaintainerImpl(aContext, aBuddyList, aLaptopBTs, aDesktopBTs,
		aPDABTs, aCellNames) );
	ret->ConstructL();
	return ret.release();
}
